/* 
 * File:   Mp3_05.c
 * FM/AMWI with MP3 Encoder&Decoder
 * Created on 2015/09/19, 19:10
 */

#include <18F26K22.h>

#device ansi
#device const=read_only
#device PASS_STRINGS=IN_RAM
#opt compress

#fuses INTRC_IO,PLLEN,NOFCMEN,NOIESO,NOBROWNOUT,NOWDT,NOPBADEN,NOMCLR,STVREN,NOLVP,NOXINST,NOCPB,NOCPD,NODEBUG
//#use delay (clock=1000000)
#use I2C(MASTER,FORCE_SW,SDA=PIN_A5,SCL=PIN_A4,SLOW)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "integer.h"
#include "diskio.h"
#include "ffconf.h"
#include "ff.h"
#include "user.h"

#define RECORDEDFILELIST    "/RECFIL05.DAT"
#define RECLEN_RECORDEDFILE 64UL
#define RF_FILENAME         0
#define RF_FILESIZE         13
#define RF_RECSTARTDT       17
#define RF_RECSTOPDT        23
#define RF_TOTALSEC         29
#define RF_STATIONNAME      33
#define RF_FREQUENCY        47

#define	ENCPATHESFILE       "/ENCPAT05.DAT"     // VS1063A PATCHES / Encoding only, compressed plugin
#define	PATHESFILE          "/PATCH05.DAT"      // VS1063A PATCHES / Full Patch
#define STATIONLISTFILE     "/STATIN05.TXT"     // station list
#define SCHEDULEFILE        "/SCHDL05.TXT"      // record schedule

#define LOGFILE             "/LOG05.TXT"

//#define CHATTERING_ADJUST   8       // 8
#define CHATTERING_ADJUST   4       // 8
#define VOLUME_STEP         4

#define ENCODE_BUF_SIZE         (_MIN_SS * 2)
#define ENCODE_BUF_SIZE_HALF    (_MIN_SS)
#define MP3FILENAME_BUF_LEN     13
#define STATIONNAME_BUF_LEN     14

#define NUM_MAX_STATIONS    10
#define RADIO_BAND_FM       1
#define RADIO_BAND_AM       2
#define SCHEDULED_REC_SPEAKER_ON        '1'
#define SCHEDULED_REC_SPEAKER_OFF       '0'

/* PIC Pin definitions */
// VS1063
#define MP3_SO          PIN_B2
#define MP3_DREQ        PIN_C6
#define MP3_SI          PIN_B3
#define MP3_CS          PIN_B0
#define MP3_DCS         PIN_C7
#define MP3_SCLK        PIN_B1
#define MP3_RESET       PIN_B4
// SD card slot
#define SD_CS           PIN_C2          // SD card/CS
// Si4735
#define SI_RST          PIN_B5          // Si4735/RSTB
// SWITCH
#define SW_PLAYREC      PIN_E3          // Recording start&stop / Playback start&stop / Fix adjusted datetime at RTC adjust mode
#define SW_MODE         PIN_A2          // Change mode (FM -> AM -> Playback -> RTC adjust)
#define SW_NEXT         PIN_A0          // Next radio station / Select recorded data / Cursor move to right at RTC adjust mode
#define SW_PREV         PIN_A1          // Prev radio station / Select recorded data / Cursor move to left at RTC adjust mode
#define SW_UP           PIN_B7          // Volume up / Increment a digit at RTC adjust mode
#define SW_DOWN         PIN_B6          // Volume down / Decrement a digit at RTC adjust mode
// POWER
#define PIN_POWER_ON_LV         PIN_A6      // keep low voltage (output)
#define PIN_POWER_ON_HV         PIN_A7      // keep high voltage (output)
#define PIN_DETECT_POWER_OFF    PIN_C0      // detect power off or RTC alarm (input)
// LCD
//#define LCD_RST         PIN_A3        // LCD's reset pin is fixed to VCC.
// LED
#define LED             PIN_C1

#define sw_playrec()            input(SW_PLAYREC)
#define sw_mode()               input(SW_MODE)
#define sw_next()               input(SW_NEXT)
#define sw_prev()               input(SW_PREV)
#define sw_up()                 input(SW_UP)
#define sw_down()               input(SW_DOWN)
#define sw_detectpoff()         input(PIN_DETECT_POWER_OFF)

#define SWPLAYREC_PRESSED       0x80
#define SWMODE_PRESSED          0x81
#define SWNEXT_PRESSED          0x82
#define SWPREV_PRESSED          0x83
#define SWUP_PRESSED            0x84
#define SWDOWN_PRESSED          0x85
#define SWPOFF_PRESSED          0x86
#define RECPREP_ALARMED         0x87
#define RECSTART_ALARMED        0x88
#define RECSTOP_ALARMED         0x89
#define CHANGE_TO_FM            0x8A
#define CHANGE_TO_AM            0x8B
#define CHANGE_TO_RADIO_REC     0x8C
#define NO_RESPONSE_VS1063      0x8D

#define EEPROM_FM_STATIONS      0x000
#define EEPROM_AM_STATIONS      0x001
#define EEPROM_ALARM_NO         0x002
#define EEPROM_ENC_SETTING_NO   0x003
#define EEPROM_FM_START         0x010
#define EEPROM_AM_START         0x0B0
#define EEPROM_LASTSTART_DT     0x150
#define EEPROM_LAST_VOLUME      0x156
#define EEPROM_REC_SCHEDULE1    0x160
#define EEPROM_REC_SCHEDULE2    0x170
#define EEPROM_REC_SCHEDULE2_FMAM       (EEPROM_REC_SCHEDULE2 + 6)
#define EEPROM_REC_SCHEDULE2_FREQ_LB    (EEPROM_REC_SCHEDULE2 + 7)
#define EEPROM_REC_SCHEDULE2_FREQ_UB    (EEPROM_REC_SCHEDULE2 + 8)
#define EEPROM_REC_SCHEDULE2_FINDEX     (EEPROM_REC_SCHEDULE2 + 9)
#define EEPROM_REC_SCHEDULE2_SPEAKER    (EEPROM_REC_SCHEDULE2 + 10)
#define EEPROM_REC_SCHEDULE2_ENCSET     (EEPROM_REC_SCHEDULE2 + 11)
#define EEPROM_REC_SCHEDULE3    0x180
#define EEPROM_SI4735_REV       0x190
#define	EEPROM_DEBUG_1          0x1A0
#define	EEPROM_DEBUG_2          0x1B0


typedef enum {
	MODE_FM = 0,            // (0) FM radio mode 
	MODE_AM,                // (1) AM radio mode 
    MODE_PLAYBACK,          // (2) Playback mode 
    MODE_COUNT
} MODE;

typedef enum {
    SUBMODE_ADJUST_RTC = 0,
    SUBMODE_ENCODE_SETTING,    // (1) change encode setting 
    SUBMODE_CONFIRM_SCHEDULE,  // (2) Confirm recording schedule 
    SUBMODE_RETURN,            // (3) Sub mode ends
    SUBMODE_COUNT
} SUBMODE;

typedef struct reccond {
    uint8_t     FmAm;                   // FM=1, AM=2
    uint16_t    Freq;                   // Frequency (Unit: FM=10KHz, AM=1KHz)
    char        SpeakerOn;              // '1':On '0':Off
    uint8_t     EncSet;                 // Encode Setting (index of ClockfTab array etc in vs1063.c)
    char        StName[STATIONNAME_BUF_LEN];             // station name
    uint8_t     Index;                  // index of station list
} RECCONDITION;

static FATFS        Fatfs;

static uint8_t      EVENT_SWANY;
#define      EVENT_SWPLAYREC    0x01
#define      EVENT_SWMODE       0x02
#define      EVENT_SWNEXT       0x03
#define      EVENT_SWPREV       0x04
#define      EVENT_SWUP         0x05
#define      EVENT_SWDOWN       0x06
#define      EVENT_SWPOFF       0x07
#define      EVENT_RECPREP      0x08
#define      EVENT_RECSTART     0x09
#define      EVENT_RECSTOP      0x0A

static uint16_t     CHATTERING_TMR;
static uint16_t     SW_CONTINUE_COUNT;
static BOOLEAN      SwmodeLast;

static uint16_t     Volume;
static BOOLEAN      SdExistsFlg;

#define RTC_POWERON             1
#define MANUAL_POWERON          0
static BOOLEAN      PowerOnTypeFlg;

static BOOLEAN      InScheduledRecFlg;      // In Scheduled Recording Process

// arg of SetPicClock()
#define PIC_CLOCK_HIGHEST   3       // 
#define PIC_CLOCK_HIGH      2       // encode
#define PIC_CLOCK_MEDIUM    1       // decode
#define PIC_CLOCK_NORMAL    0       // radio

// arg of SetRecordingAlarmToRTC()
#define POWERON_AND_PREPARATION     1
#define RECORDING_START             2
#define RECORDING_STOP              3

static uint16_t     WaitMsCycle;
static uint16_t     Timer0InitValue;

static FIL         FileLog;

void    InitApp(FIL *file);
void    InitEncoderDecoder(FIL *file);
void    SetPicClock(uint8_t level);
void    StartupDateTime();
void    ShutDown();
int8    PauseAndSelectItem();
int8    RadioMode(FIL *file, uint8_t fmam, RECCONDITION *reccond);
int8    ExecRecording(FIL *file, uint8_t EncSet, char *stname, uint16_t freq);
uint8_t GetFmBlendMonoThreshold(int StereoMode);
int8    PlaybackMode(FIL *file);
int8    SelectEncodeSetting();
int8    AdjustRtc();
// Plugin data
void    LoadPluginFromFile(char *PluginFile, FIL *file);
void    WritePluginToEEPROM(char *PluginFile, FIL *file);
void    CheckPluginInEEPROM(char *PluginFile, FIL *file);
void    CheckPluginInEEPROM2(char *PluginFile, FIL *file);
void    LoadPluginFromEEPROM();
int8    SubMode();
int8    SelectSubMode();
// Recorded file
int8    GetNextMp3FileName(char *Mp3File, FIL *file);
void    CreateRecordedFileList(FIL *file);
uint16_t AddRecordedFileToList(FIL *file, char *Mp3File, uint8_t *recstartdt, char *stname, uint16_t freq);
void    SaveMp3Filesize(FIL *file, uint16_t recno, uint32_t filesize);
void    SaveRecordingStopDatetime(FIL *file, uint16_t recno, uint8_t *recstopdt);
uint16_t GetNumOfRecordedFiles(FIL *file);
int     GetRecordedFileName(FIL *file, uint16_t fileno, char *Mp3File);
// Scheduled recording
int8    ConfirmRecordingSchedule();
int8    SetNextScheduleToEEPROM(FIL *file);
int8    SetRecordingAlarmToRTC(uint8_t alarmno);
BOOLEAN IsRtcPowerOn();
int8    OnRTCAlarm();
int8    GetCurrentScheduleFromEEPROM(RECCONDITION *recconf);
// Station list
int8    SetStationList(FIL *file);
uint8_t GetFrequencyListIndex(uint8_t fmam, uint16_t freq);
int8    GetFrequencyAndStationName(uint8_t fmam, uint8_t index, uint16_t *freq, char stname[]);
// Utility functions
int8    ConvNumericStrToBCD(char *numstr, uint8_t *bcd);
int8    CompareDateBCD(uint8_t *date1, uint8_t *date2);
void    ConvBCDtoStr(uint8_t *bcd, uint8_t len, char *buf);
uint16_t Int2Bcd(uint16_t i);
uint16_t Bcd2Int(uint16_t b);
void    wait_ms(uint16_t ms);
void    WriteLog(char *buff, int dtout);
void    SdReadWriteTest();

// CCS-C compiler does not support multi source compile.
#include "spi_sd.c"
#include "ff.c"
//#include "at24c128.c"
#include "m41t62.c"
#include "lcd.c"
#include "si4735.c"
#include "vs1063.c"


/*-----------------------------------------------------------------------*/
/* main function                                                         */
/*-----------------------------------------------------------------------*/
int main(int argc, char** argv) {
    FRESULT     res;
    FIL         File;
    int8        ret;
    MODE        Mode;
    RECCONDITION        reccond;

    InitApp(&File);
    EVENT_SWANY = 0;
    CHATTERING_TMR = 0;
    SwmodeLast = sw_mode();
    memset(&reccond, 0, sizeof(RECCONDITION));
    Mode = MODE_FM;
    
    LCD_LPrintln(0, "MP3_05 start !  ");
    if (IsRtcPowerOn() == 1){
        LCD_LPrintln(0, "ALARM BOOT      ");
        PowerOnTypeFlg = RTC_POWERON;
        if (GetCurrentScheduleFromEEPROM(&reccond) == 0){
            if (reccond.FmAm == RADIO_BAND_FM) Mode = MODE_FM;
            else Mode = MODE_AM;
            SetRecordingAlarmToRTC(RECORDING_START);      // set recording start alarm to RTC
            InScheduledRecFlg = 1;          // scheduled recording starts here
            if (reccond.SpeakerOn == SCHEDULED_REC_SPEAKER_ON){
                output_high(PIN_POWER_ON_HV);
            }
        } else {
            WriteLog("No schedule EEPROM", 0);
            ShutDown();
        }
    } else {        // Normal Power On
        output_high(LED);
        PowerOnTypeFlg = MANUAL_POWERON;
        LCD_LPrintln(1, "Normal Start    ");
        output_high(PIN_POWER_ON_HV);
    }
    if (SdExistsFlg == 1){
        //SdReadWriteTest();
        LCD_LPrintln(0, "Set Station List");
        SetStationList(&File);
        LCD_LPrintln(0, "Create Rec File ");
        CreateRecordedFileList(&File);
        if (PowerOnTypeFlg == MANUAL_POWERON){
            LCD_LPrintln(0, "Read Schedule   ");
            SetNextScheduleToEEPROM(&File);
        }
    } else {
        write_eeprom(EEPROM_DEBUG_1, res);
        LCD_LPrintln(0, "No SD Card      ");
    }
    if (PowerOnTypeFlg == MANUAL_POWERON){
        SetRecordingAlarmToRTC(POWERON_AND_PREPARATION);      // set power on alarm to RTC
    }
    SetPicClock(PIC_CLOCK_NORMAL);

    /* Main Loop */
    while (1){
        switch (Mode){
            case MODE_FM:
                ret = RadioMode(&File, RADIO_BAND_FM, &reccond);
                break;
            case MODE_AM:
                ret = RadioMode(&File, RADIO_BAND_AM, &reccond);
                break;
            case MODE_PLAYBACK:
                ret = PlaybackMode(&File);
                break;
        }
        if (ret == SWMODE_PRESSED){
            Mode++;
            if (Mode == MODE_COUNT){
                Mode = MODE_FM;
            }
        } else if (ret == SWPOFF_PRESSED){
            break;
        } else if (ret == CHANGE_TO_RADIO_REC){
            if (GetCurrentScheduleFromEEPROM(&reccond) == 0){
                if (reccond.FmAm == RADIO_BAND_FM) Mode = MODE_FM;
                else Mode = MODE_AM;
                SetRecordingAlarmToRTC(RECORDING_START);      // set recording start alarm to RTC
                InScheduledRecFlg = 1;          // scheduled recording starts here
            }
        } else if (ret == NO_RESPONSE_VS1063){
            WriteLog("NO_RESPONSE_VS1063 -> HW Reset", 0);
            InitEncoderDecoder(&File);
        }
    }

    ShutDown();
    return (0);
}

#if 0
void SdReadWriteTest(){
    FIL          Fil1;
    FIL          Fil2;
    FRESULT      res;
    UINT         br, br2;
    uint8_t     buff[8];

    res = f_open(&Fil1, "/A.TXT", FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_open(&Fil2, "/B.TXT", FA_WRITE | FA_OPEN_ALWAYS);
        if (res == FR_OK){
            while (1){
                res = f_read(&Fil1, buff, 8, &br);
                if (res != FR_OK || br == 0) break;
                f_write(&Fil2, buff, br, &br2);
            }
            f_close(&Fil2);
        }
        f_close(&Fil1);
    }
}
#endif

void InitApp(FIL *file){
    FRESULT     res;
    
    /* Initialize peripherals */
    setup_adc(ADC_OFF);         // ADC disabled
    setup_ccp1(CCP_OFF);        // CCP1 disabled
    setup_ccp2(CCP_OFF);        // CCP2 disabled

    SetPicClock(PIC_CLOCK_HIGH);
    
    output_high(PIN_POWER_ON_LV);
    output_low(PIN_POWER_ON_HV);
    output_float(PIN_DETECT_POWER_OFF);

    output_float(SW_PLAYREC);
    output_float(SW_MODE);
    output_float(SW_NEXT);
    output_float(SW_PREV);
    output_float(SW_UP);
    output_float(SW_DOWN);
    output_float(MP3_DREQ);   // DREQ
    port_b_pullups(0xC0);     // B6,B7
    //port_b_pullups(TRUE);

    output_low(MP3_DCS);
    output_high(MP3_CS);
    //output_low(LCD_RST);

    //SPI2 controls VS1063
    setup_spi2(SPI_MASTER | SPI_CLK_DIV_16 | SPI_SCK_IDLE_LOW | SPI_XMIT_L_TO_H | SPI_SAMPLE_AT_MIDDLE);    // for 32MHz
    //setup_spi2(SPI_MASTER | SPI_CLK_DIV_4 | SPI_SCK_IDLE_LOW | SPI_XMIT_L_TO_H | SPI_SAMPLE_AT_MIDDLE);
    //setup_spi2(SPI_MASTER | SPI_CLK_DIV_64 | SPI_SCK_IDLE_LOW | SPI_XMIT_L_TO_H | SPI_SAMPLE_AT_MIDDLE);

    /* Enable interrupts */
#if 0
    setup_timer_1(T1_INTERNAL | T1_DIV_BY_8);
    set_timer1(Timer1InitValue);
    clear_interrupt(INT_TIMER1);
    enable_interrupts(INT_TIMER1);
#endif
    setup_timer_0(T0_INTERNAL | T0_DIV_16);
    set_timer0(Timer0InitValue);
    clear_interrupt(INT_TIMER0);
    enable_interrupts(INT_TIMER0);
    
    enable_interrupts(GLOBAL);

    while (LCD_Init() != 0) ;
    //AT24C128_Test();
    LCD_LPrintln(0, "SD Card Init    ");
    res = f_mount(&Fatfs, _T("0:"), 1);
    if (res == FR_OK){
        DWORD   nclst;
        FATFS   *fs;
        DWORD   freesize;
        char    freesizestr[8];
        // checks free area
        if (f_getfree(_T("0:"), &nclst, &fs) == FR_OK){
            freesize = nclst * 100 / (fs->n_fatent - 2);
            itoa(freesize, 10, freesizestr);
            LCD_LPrintln(1, "Free area        ");
            LCD_LCPrintln(1, 11, freesizestr);
            LCD_LCPrintln(1, 15, "%");
            if (freesize < 50) wait_ms(5000);
        }
        SdExistsFlg = 1;
        f_open(&FileLog, LOGFILE, FA_READ | FA_WRITE | FA_OPEN_ALWAYS);
        f_lseek(&FileLog, f_size(&FileLog));
    } else {
        SdExistsFlg = 0;
        LCD_LPrintln(0, "NO SD Card      ");
        wait_ms(10000);
        ShutDown();
    }
    
    LCD_LPrintln(0, "RTC Init        ");
    //RTC_Reset();
    RTC_Init();
    StartupDateTime();
    
    LCD_LPrintln(0, "VS1063 HW Reset ");
    Volume = 0x40;
    InitEncoderDecoder(file);
}

void InitEncoderDecoder(FIL *file){
    uint8_t     encset;
    
    VS1063_HardwareReset();
    if (SdExistsFlg == 1 && f_stat(ENCPATHESFILE, NULL) == FR_OK){
        //WritePluginToEEPROM(ENCPATHESFILE, file);
        //CheckPluginInEEPROM(ENCPATHESFILE, file);
        //CheckPluginInEEPROM2(ENCPATHESFILE, file);
        LCD_LPrintln(0, "Load Patch SD   ");
        LoadPluginFromFile(ENCPATHESFILE, file);
    } else {
        //LCD_LPrintln(0, "Load Patch ROM  ");
        //LoadPluginFromEEPROM();
        LCD_LPrintln(0, "No VS1063 Patch ");
        LCD_LPrintln(1, "in EEPROM       ");
        wait_ms(10000);
        ShutDown();
    }
    
    // GR[hݒԍ͈̔̓`FbN
    encset = read_eeprom(EEPROM_ENC_SETTING_NO);
    if (!(encset >= 0 && encset < NUMENCODESETTINGS)){
        write_eeprom(EEPROM_ENC_SETTING_NO, 0);
    }
}

void SetPicClock(uint8_t level){
    switch (level){
    case PIC_CLOCK_HIGHEST:
        setup_oscillator(OSC_64MHZ);
        
        // (1000us)/(4/64MHz)*250 = 64
        // -> 64*delay_cycle(250) = 1ms
        WaitMsCycle = 64;
        
        // Timer0 : (4/64MHz) * 16(prescaler) = 1us
        // 1us * 65536 = 65ms
        Timer0InitValue = 0;
        
        break;
            
    case PIC_CLOCK_HIGH:    
        setup_oscillator(OSC_32MHZ);
        
        // (1000us)/(4/32MHz)*250 = 32
        // -> 32*delay_cycle(250) = 1ms
        WaitMsCycle = 32;
        
        // Timer0 : (4/32MHz) * 16(prescaler) = 2us
        // 2us * 65536 = 131ms
        Timer0InitValue = 0;
        
        // Timer1 : (4/32MHz) * 8(prescaler) = 1 us
        // 1us * 65536 = 65ms
        //Timer1InitValue = 0;
        
        // Timer0 : (4/32MHz) * 8(prescaler) = 1 us
        //Timer0InitValue = (65536 - 62500);
        
        break;
    case PIC_CLOCK_MEDIUM:
        setup_oscillator(OSC_8MHZ);
        
        // (1000us)/(4/8MHz)*250 = 8
        // -> 8*delay_cycle(250) = 1ms
        WaitMsCycle = 8;
        
        // Timer0 : (4/8MHz) * 16(prescaler) = 8us
        // 8us * (65536/4) = 131ms
        Timer0InitValue = 49152;    // = 65536 - (65536 / 4)
        
        // Timer1 : (4/8MHz) * 8(prescaler) = 4 us
        // 4us * (65536/4) = 65ms
        //Timer1InitValue = 49152;    // = 65536 - (65536 / 4)
        
        // Timer0 : (4/8MHz) * 8(prescaler) = 4 us
        // 128 us * 15625 = 2s
        //Timer0InitValue = (65536 - 15625);
        
        break;
    case PIC_CLOCK_NORMAL:
        setup_oscillator(OSC_1MHZ);
        
        // (1000us)/(4/1MHz)*250 = 1
        // -> 1*delay_cycle(250) = 1ms
        WaitMsCycle = 1;
        
        // Timer0 : (4/1MHz) * 16(prescaler) = 64us
        // 64us * (65536/32) = 131ms
        Timer0InitValue = 63488;    // = 65536 - (65536 / 32)
        
        // Timer1 : (4/1MHz) * 8(prescaler) = 32 us
        // 32us * (65536/32) = 65ms
        //Timer1InitValue = 63488;    // = 65536 - (65536 / 32)
        
        // Timer0 : (4/1MHz) * 8(prescaler) = 32 us
        // 1024 us * 1953 = 2s
        //Timer0InitValue = (65536 - 1953);
        
        break;
    }
}


/*-----------------------------------------------------------------------*/
/* Save Current Date&Time to EEPROM and display                          */
/*-----------------------------------------------------------------------*/
void StartupDateTime(){
    uint8_t     dt[6], i;
    char        buff[17];
    
    RTC_GetCurrentDateTimeBCD(dt);
    for (i=0; i<6; i++){
        write_eeprom(EEPROM_LASTSTART_DT + i, dt[i]);
    }
    
    buff[0] = itoc(dt[0] >> 4);		// YY
    buff[1] = itoc(dt[0] & 0x0F);
    buff[2] = '/';
    buff[3] = itoc(dt[1] >> 4);		// MM
    buff[4] = itoc(dt[1] & 0x0F);
    buff[5] = '/';
    buff[6] = itoc(dt[2] >> 4);		// DD
    buff[7] = itoc(dt[2] & 0x0F);
    buff[8] = ' ';
    buff[9] = itoc(dt[3] >> 4);		// HH
    buff[10] = itoc(dt[3] & 0x0F);
    buff[11] = ':';
    buff[12] = itoc(dt[4] >> 4);		// MM
    buff[13] = itoc(dt[4] & 0x0F);
    buff[14] = ' ';
    buff[15] = ' ';
    buff[16] = '\0';
    
    LCD_LPrintln(1, buff);
}

/*-----------------------------------------------------------------------*/
/* Power down                                                            */
/*-----------------------------------------------------------------------*/
void ShutDown(){
    f_close(&FileLog);
    output_low(PIN_POWER_ON_LV);
}


/*-----------------------------------------------------------------------*/
/* Radio Mode                                                            */
/*  Line-0 : Station Name                                                */
/*  Line-1 : Current Time/Status/Error Info                              */
/*-----------------------------------------------------------------------*/
int8 RadioMode(FIL *file, uint8_t FmAm, RECCONDITION *reccond){
    int8                ret;
    uint16_t            freq;
    char                stname[STATIONNAME_BUF_LEN];
    static uint8_t      CurFreqIndex = 0;
    uint8_t             NumStations;
    uint8_t             EncSet;
    
    LCD_Icon_RadioMode(1);
    
    if (FmAm == RADIO_BAND_FM){      // FM
        NumStations = read_eeprom(EEPROM_FM_STATIONS);
        if (InScheduledRecFlg == 1){    // Scheduled recording
            CurFreqIndex = reccond->Index;
            EncSet = reccond->EncSet;
        } else {
            if (!(CurFreqIndex >= 0 && CurFreqIndex < NumStations)) CurFreqIndex = 0;
            EncSet = read_eeprom(EEPROM_ENC_SETTING_NO);
        }
        GetFrequencyAndStationName(FmAm, CurFreqIndex, &freq, stname);
        Si4735_InitFm(GetFmBlendMonoThreshold(GetStereoModeOfEncSet(EncSet)));
        Si4735_TuneFrequencyFm(freq);
    } else {        // AM
        NumStations = read_eeprom(EEPROM_AM_STATIONS);
        if (InScheduledRecFlg == 1){    // Scheduled recording
            CurFreqIndex = reccond->Index;
        } else {
            if (!(CurFreqIndex >= 0 && CurFreqIndex < NumStations)) CurFreqIndex = 0;
        }
        GetFrequencyAndStationName(FmAm, CurFreqIndex, &freq, stname);
        Si4735_InitAm();
        Si4735_TuneFrequencyAm(freq);
    }
    write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
    
    ret = VS1063_MonitorMode();
    if (ret == NO_RESPONSE_VS1063){
        return ret;
    }
    LCD_RadioMode_Line0(FmAm, stname);
    
    //TODO:LINE-1
    LCD_LPrintln(1, LCD_BLANK_LINE);
    
    while (1){
        
        ret = PauseAndSelectItem();
        if (ret == SWPLAYREC_PRESSED){      // ^蓮X^[g
            if (SdExistsFlg == 1){
                LCD_Recording_Line1("START          ");
                ret = ExecRecording(file, read_eeprom(EEPROM_ENC_SETTING_NO), stname, freq);
                if (ret == SWPLAYREC_PRESSED){
                    LCD_Recording_Line1("STOP            ");
                    ret = VS1063_MonitorMode();
                } else if (ret == SWPOFF_PRESSED){
                    break;
                } else if (ret != 0){       // error
                    RTC_ClearAlarm();
                    LCD_Recording_Line1("REC ERROR       ");
                }
            }
            continue;
        } else if (ret == SWMODE_PRESSED){
            if (InScheduledRecFlg == 0){
                if (SW_CONTINUE_COUNT == 0){            // Change to Playback mode
                    break;
                } else {        // 
                    ret = SubMode();
                    if (ret == SWPOFF_PRESSED){
                        break;
                    }
                    LCD_RadioMode_Line0(FmAm, stname);
                    LCD_LPrintln(1, LCD_BLANK_LINE);
                }
            }
        } else if (ret == SWNEXT_PRESSED){      // next station
            if (InScheduledRecFlg == 0){
                CurFreqIndex++;
                if (CurFreqIndex == NumStations) CurFreqIndex = 0;
                GetFrequencyAndStationName(FmAm, CurFreqIndex, &freq, stname);
                LCD_RadioMode_Line0(FmAm, stname);
                if (FmAm == RADIO_BAND_FM) Si4735_TuneFrequencyFm(freq);
                else Si4735_TuneFrequencyAm(freq);
                continue;
            }
        } else if (ret == SWPREV_PRESSED){      // previous station
            if (InScheduledRecFlg == 0){
                if (CurFreqIndex == 0) CurFreqIndex = NumStations - 1;
                else CurFreqIndex--;
                GetFrequencyAndStationName(FmAm, CurFreqIndex, &freq, stname);
                LCD_RadioMode_Line0(FmAm, stname);
                if (FmAm == RADIO_BAND_FM) Si4735_TuneFrequencyFm(freq);
                else Si4735_TuneFrequencyAm(freq);
                continue;
            }
        } else if (ret == SWUP_PRESSED){        // Volume up
            if (Volume > VOLUME_STEP){
                Volume -= VOLUME_STEP;
                VS1063_SetVolume(Volume);
                write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
            }
            continue;
        } else if (ret == SWDOWN_PRESSED){      // Volume down
            if (Volume < (0x00FF - VOLUME_STEP)){
                Volume += VOLUME_STEP;
                VS1063_SetVolume(Volume);
                write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
            }
            continue;
        } else if (ret == SWPOFF_PRESSED){      // power off
            break;
        } else if (ret == RECPREP_ALARMED){
            LCD_Recording_Line1("RTC REC PREP    ");
            if (GetCurrentScheduleFromEEPROM(reccond) == 0){
                //DebugRTCAlarm();
                if (FmAm != reccond->FmAm){
                    ret = CHANGE_TO_RADIO_REC;
                    break;
                }
                // continue radio mode
                InScheduledRecFlg = 1;          // sheduled recording starts
                SetRecordingAlarmToRTC(RECORDING_START);      // set start alarm to RTC
                CurFreqIndex = reccond->Index;
                GetFrequencyAndStationName(FmAm, CurFreqIndex, &freq, stname);
                if (FmAm == RADIO_BAND_FM){
                    Si4735_FmBlendMonoThreshold(GetFmBlendMonoThreshold(GetStereoModeOfEncSet(reccond->EncSet)));
                    Si4735_TuneFrequencyFm(freq);
                } else Si4735_TuneFrequencyAm(freq);
                LCD_RadioMode_Line0(FmAm, stname);
            }
        } else if (ret == RECSTART_ALARMED){        // Scheduled "Encoding" start
            if (SdExistsFlg == 1){
                LCD_Recording_Line1("RTC START       ");
                SetRecordingAlarmToRTC(RECORDING_STOP);      // set stop alarm to RTC
                ret = ExecRecording(file, reccond->EncSet, stname, freq);
                InScheduledRecFlg = 0;          // scheduled recording ends here
                if (ret == SWPLAYREC_PRESSED){
                    LCD_Recording_Line1("STOP            ");
                    SetNextScheduleToEEPROM(file);    // next schedule
                    SetRecordingAlarmToRTC(POWERON_AND_PREPARATION);      // set power on alarm to RTC
                    Si4735_FmBlendMonoThreshold(GetFmBlendMonoThreshold(GetStereoModeOfEncSet(read_eeprom(EEPROM_ENC_SETTING_NO))));
                    ret = VS1063_MonitorMode();
                } else if (ret == RECSTOP_ALARMED){
                    LCD_Recording_Line1("RTC STOP        ");
                    SetNextScheduleToEEPROM(file);    // next schedule
                    SetRecordingAlarmToRTC(POWERON_AND_PREPARATION);      // set power on alarm to RTC
                    if (PowerOnTypeFlg == RTC_POWERON && reccond->SpeakerOn == SCHEDULED_REC_SPEAKER_OFF){
                        ret = SWPOFF_PRESSED;
                        break;
                    }
                    Si4735_FmBlendMonoThreshold(GetFmBlendMonoThreshold(GetStereoModeOfEncSet(read_eeprom(EEPROM_ENC_SETTING_NO))));
                    ret = VS1063_MonitorMode();                    
                } else if (ret == SWPOFF_PRESSED){
                    break;
                } else if (ret != 0){
                    RTC_ClearAlarm();
                    LCD_Recording_Line1("REC ERROR       ");
                }
            }
            continue;
        }
    }
    
    //disable_interrupts(INT_TIMER0);
    Si4735_PowerDown();
    LCD_Icon_RadioMode(0);
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Recording                                                             */
/*-----------------------------------------------------------------------*/
int8 ExecRecording(FIL *file, uint8_t EncSet, char *stname, uint16_t freq){
    char    Mp3FileName[MP3FILENAME_BUF_LEN];
    int8    ret;
    uint8_t datetime[6];
    uint16_t    recno;
    
    //SetPicClock(PIC_CLOCK_HIGHEST);
    SetPicClock(PIC_CLOCK_HIGH);
    ret = GetNextMp3FileName(Mp3FileName, file);
    if (ret == 0){
        RTC_GetCurrentDateTimeBCD(datetime);                // recording start datetime
        recno = AddRecordedFileToList(file, Mp3FileName, datetime, stname, freq);
        ret = VS1063_Encode(Mp3FileName, file, EncSet, recno);
        RTC_GetCurrentDateTimeBCD(datetime);                // recording stop datetime
        SaveRecordingStopDatetime(file, recno, datetime);
    }
    SetPicClock(PIC_CLOCK_NORMAL);
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Get FM Blend Mono Threshold Value                                     */
/*  StereoMode : 1=Stereo 0=Mono                                         */
/*-----------------------------------------------------------------------*/
uint8_t GetFmBlendMonoThreshold(int StereoMode){
    if (StereoMode <= 1) return 0;
    else return 127;
}

/*-----------------------------------------------------------------------*/
/* Playback Encorded sound                                               */
/*  line-0 : Record date & time                                          */
/*  line-1 : Station name or Status                                      */
/*-----------------------------------------------------------------------*/
int8 PlaybackMode(FIL *file){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    char        line[17];
    uint16_t    num;
    uint16_t    currec;
    uint16_t    lastrec;
    UINT        br;
    uint32_t    ofs;
    int8        ret;
    int         continue_flag;
    BOOLEAN     execdecflg;
    uint32_t    totalsec;
    uint32_t    filesize;
    uint16_t    bytespersec;
    
    ret = 0;
    execdecflg = 0;
    LCD_Icon_PlaybackMode(1);
    
    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
        if (res == FR_OK && br == RECLEN_RECORDEDFILE){
            num = *((uint16_t *)&buff[0]);      // Num of recorded files
            lastrec = *((uint16_t *)&buff[2]);
            if (lastrec >= 1 && lastrec <= num){
                currec = lastrec;
            } else {
                currec = num;
            }
        } else {
            num = 0;
        }
        if (num == 0){
            f_close(file);
            LCD_LPrintln(0, "No recorded file");
            wait_ms(2000);
            continue_flag = 0;
            ret = SWMODE_PRESSED;
        } else {
            continue_flag = 1;
        }
        //currec = num;
        while (continue_flag){
            ofs = (uint32_t)currec * (uint32_t)RECLEN_RECORDEDFILE;
            res = f_lseek(file, ofs);
            if (res == FR_OK){
                res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
                if (res == FR_OK && br == RECLEN_RECORDEDFILE){
                    memset(line, ' ', 16);
                    strncpy(line, buff, 6);     // [0]-[5] : YYMMDD
                    line[7] = itoc(buff[RF_RECSTARTDT + 3] >> 4);
                    line[8] = itoc(buff[RF_RECSTARTDT + 3] & 0x0F);     // hour
                    line[9] = itoc(buff[RF_RECSTARTDT + 4] >> 4);
                    line[10] = itoc(buff[RF_RECSTARTDT + 4] & 0x0F);    // min
                    line[16] = '\0';
                    LCD_LPrintln(0, line);

                    memset(line, ' ', 16);
                    line[16] = '\0';
                    LCD_LPrintln(1, &buff[RF_STATIONNAME]);
                    
                    memcpy(&totalsec, &buff[RF_TOTALSEC], 4);
                    memcpy(&filesize, &buff[RF_FILESIZE], 4);
                    if (totalsec != 0){
                        bytespersec = filesize / totalsec;
                    } else {
                        bytespersec = 0;
                    }
                    
                    // select
                    ret = PauseAndSelectItem();
                    if (ret == SWPLAYREC_PRESSED){          // start playback
                        f_close(file);
                        SetPicClock(PIC_CLOCK_MEDIUM);
                        ret = VS1063_Decode(buff, file, bytespersec);
                        execdecflg = 1;
                        SetPicClock(PIC_CLOCK_NORMAL);
                        if (ret == CHANGE_TO_RADIO_REC){
                            continue_flag = 0;
                        } else if (ret == SWPOFF_PRESSED){      // power off
                            continue_flag = 0;
                        }
                        f_open(file, RECORDEDFILELIST, FA_READ | FA_OPEN_EXISTING);     // re-open
                    } else if (ret == SWMODE_PRESSED){      //
                        continue_flag = 0;
                    } else if (ret == SWNEXT_PRESSED){      // next data
                        if (currec < num) currec++;
                        else currec = 1;
                    } else if (ret == SWPREV_PRESSED){      // previous data
                        if (currec == 1) currec = num;
                        else currec--;
                    } else if (ret == SWPOFF_PRESSED){      // power off
                        continue_flag = 0;
                    }
                }
            }
        }
        f_close(file);
    }
    
    if (execdecflg == 1 && ret != SWPOFF_PRESSED){
        // PlaybackRadioւ̕AłƂǂVS1063sɂȂ̂Ŗ񃊃Zbg
        InitEncoderDecoder(file);
    }
    LCD_Icon_PlaybackMode(0);
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Select Encode Setting                                                 */
/*-----------------------------------------------------------------------*/
int8 SelectEncodeSetting(){
    int8        ret;
    uint8_t     CurEncSet;
    char        label1[17];
    char        buff[17];
    
    ret = 0;
    CurEncSet = read_eeprom(EEPROM_ENC_SETTING_NO);
    
    while (1){
        // make line 0
        itoa(CurEncSet, 10, buff);
        strcpy(label1, buff);
        strcat(label1, ":");
        // SCI_CLOCKF
        switch (ClockfTab[CurEncSet]){
            case 0xE000: strcat(label1, "x5.0 "); break;
            case 0xC000: strcat(label1, "x4.5 "); break;
            case 0xA000: strcat(label1, "x4.0 "); break;
            case 0x8000: strcat(label1, "x3.5 "); break;
            case 0x6000: strcat(label1, "x3.0 "); break;
            case 0x4000: strcat(label1, "x2.5 "); break;
            default: strcat(label1, "x?? "); break;
        }
        // 
        switch (SamplerateTab[CurEncSet]){
            case 48000: strcat(label1, "SR=48.0 "); break;
            case 44100: strcat(label1, "SR=44.1 "); break;
            case 32000: strcat(label1, "SR=32.0 "); break;
            case 24000: strcat(label1, "SR=24.0 "); break;
        }
        LCD_LPrintln(0, LCD_BLANK_LINE);
        LCD_LPrintln(0, label1);
        
        // make line 1
        // Bitrate
        strcpy(label1, " BR=");
        itoa(BitrateTab[CurEncSet], 10, buff);
        strcat(label1, buff);
        strcat(label1, " ");
        //
        switch (EncFormatTab[CurEncSet]){
            case 0x60: strcat(label1, "ST       "); break;
            case 0x63: strcat(label1, "RC       "); break;
            default: break;
        }
        LCD_LPrintln(1, LCD_BLANK_LINE);
        LCD_LPrintln(1, label1);
        
        ret = PauseAndSelectItem();
        if (ret == SWPLAYREC_PRESSED){          // decide
            write_eeprom(EEPROM_ENC_SETTING_NO, CurEncSet);
            ret = SWMODE_PRESSED;
            break;
        } else if (ret == SWUP_PRESSED){        // up
            if (CurEncSet == (NUMENCODESETTINGS - 1)){
                CurEncSet = 0;
            } else {
                CurEncSet++;
            }
        } else if (ret == SWDOWN_PRESSED){      // down
            if (CurEncSet == 0){
                CurEncSet = (NUMENCODESETTINGS - 1);
            } else {
                CurEncSet--;
            }
        } else if (ret == SWMODE_PRESSED){
            write_eeprom(EEPROM_ENC_SETTING_NO, CurEncSet);
            break;
        } else if (ret == RECPREP_ALARMED){
            ret = CHANGE_TO_RADIO_REC;
            break;
        }
    }
    
    return (ret);
}


/*-----------------------------------------------------------------------*/
/* Display Recording Schedule                                            */
/*  Line-0 : start date & time , Band, SP                                */
/*  Line-1 : stop date & time, freq                                      */
/*-----------------------------------------------------------------------*/
int8 ConfirmRecordingSchedule(){
    int8        ret;
    char        buff[17];
    int8        i;
    uint8_t     b;
    uint16_t    freq;
    char        freqs[8];
    
    ret = 0;
    
    if (read_eeprom(EEPROM_REC_SCHEDULE2) == 0xFF){
        LCD_LPrintln(0, "No schedule     ");
        LCD_LPrintln(1, LCD_BLANK_LINE);
    } else {
    
        // Recording start date(YYMMDD)
        for (i=0; i<3; i++){
            b = read_eeprom(EEPROM_REC_SCHEDULE2 + i);
            buff[i*2] = itoc(b >> 4);
            buff[i*2 + 1] = itoc(b & 0x0F);
        }
        buff[6] = ' ';
        // Recording start time(HHMM)
        for (i=0; i<2; i++){
            b = read_eeprom(EEPROM_REC_SCHEDULE2 + 3 + i);
            buff[7 + i*2] = itoc(b >> 4);
            buff[7 + i*2 + 1] = itoc(b & 0x0F);
        }
        buff[11] = ' ';
        // Band (A or F)
        buff[12] = read_eeprom(EEPROM_REC_SCHEDULE2_FMAM);
        if (buff[12] == RADIO_BAND_FM) buff[12] = 'F';
        else buff[12] = 'A';
        buff[13] = ' ';
        // SP
        if (read_eeprom(EEPROM_REC_SCHEDULE2_SPEAKER) == '1'){
            buff[14] = 'S';
            buff[15] = 'P';
        } else {
            buff[14] = ' ';
            buff[15] = ' ';
        }
        buff[16] = '\0';
        LCD_LPrintln(0, buff);

        // Recording stop date(YYMMDD)
        for (i=0; i<3; i++){
            b = read_eeprom(EEPROM_REC_SCHEDULE3 + i);
            buff[i*2] = itoc(b >> 4);
            buff[i*2 + 1] = itoc(b & 0x0F);
        }
        buff[6] = ' ';
        // Recording stop time(HHMM)
        for (i=0; i<2; i++){
            b = read_eeprom(EEPROM_REC_SCHEDULE3 + 3 + i);
            buff[7 + i*2] = itoc(b >> 4);
            buff[7 + i*2 + 1] = itoc(b & 0x0F);
        }
        buff[11] = ' ';
        buff[12] = '\0';
        // Frequency
        freq = read_eeprom(EEPROM_REC_SCHEDULE2_FREQ_UB);
        freq <<= 8;
        freq |= read_eeprom(EEPROM_REC_SCHEDULE2_FREQ_LB);
        itoa(freq, 10, freqs);
        strcat(buff, freqs);
        LCD_LPrintln(1, LCD_BLANK_LINE);
        LCD_LPrintln(1, buff);
    
    }
    
    while (1){
        
        ret = PauseAndSelectItem();
        if (ret == SWPLAYREC_PRESSED){
            ret = SWMODE_PRESSED;
            break;
        } else if (ret == SWMODE_PRESSED){
            break;
        } else if (ret == RECPREP_ALARMED){
            ret = CHANGE_TO_RADIO_REC;
            break;
        }
    }
    return (ret);
}

/*-----------------------------------------------------------------------*/
/* adjust RTC                                                            */
/*  line-0 : Current Date-Time (YYMMDDWHHMMSS)                           */
/*  line-1 : adjustment value                                            */
/*-----------------------------------------------------------------------*/
int8 AdjustRtc(){
    char        date1[17];
    char        buff[17];
    uint8_t     val;
    int8        ret;
    int         continue_flag;
    uint8_t     pos;    // 0=year,1=month,2=day,3=day of week,4=hour,5=min,6=sec
    const uint8_t       startcol = 0;
    static uint8_t      loc[] = {0,2,4,6,7,9,11};
    static uint8_t      len[] = {2,2,2,1,2,2,2};
    static uint8_t      min[] = {15,1,1,1,0,0,0};
    static uint8_t      max[] = {25,12,31,7,23,59,59};
    static char         *min_s[] = {"15", "01", "01", "1", "00", "00", "00"};
    static char         *max_s[] = {"25", "12", "31", "7", "23", "59", "59"};
    
    //LCD_LPrintln(0, "Adjust RTC      ");
    //wait_ms(2000);
    
    RTC_GetCurrentDateTimeForAdjust(date1);
    ret = 0;
    pos = 0;        // year
    continue_flag = 1;
    
    while (continue_flag){
        LCD_LCPrintln(0, startcol, date1);

        // display a part of datetime
        LCD_I2C_Start();
        LCD_Locate(1, startcol);
        LCD_NPrintln(LCD_BLANK_LINE, loc[pos]);
        LCD_NPrintln(&date1[loc[pos]], len[pos]);
        LCD_Println(LCD_BLANK_LINE);
        i2c_stop();
        // convert to int
        strncpy(buff, &date1[loc[pos]], len[pos]);
        buff[len[pos]] = '\0';
        val = atoi(buff);

        while (1){
            LCD_I2C_Start();
            LCD_Locate(1, startcol + loc[pos]);
            LCD_NPrintln(&date1[loc[pos]], len[pos]);
            i2c_stop();

            ret = PauseAndSelectItem();
            if (ret == SWPLAYREC_PRESSED){
                RTC_SetDateTime(date1);
                continue_flag = 0;
                ret = SWMODE_PRESSED;
                break;
            } else if (ret == SWMODE_PRESSED){
                continue_flag = 0;
                break;
            } else if (ret == SWNEXT_PRESSED){
                if (pos < 6) pos++;
                else pos = 0;
                break;
            } else if (ret == SWPREV_PRESSED){
                if (pos > 0) pos--;
                else pos = 6;
                break;
            } else if (ret == SWUP_PRESSED){
                if (val < max[pos]){
                    val++;
                    if (pos == 3){  // day of week
                        date1[loc[pos]]++;
                    } else {
                        if (date1[loc[pos] + 1] == '9'){
                            date1[loc[pos] + 1] = '0';
                            date1[loc[pos]]++;
                        } else {
                            date1[loc[pos] + 1]++;
                        }
                    }
                } else {
                    val = min[pos];
                    strncpy(&date1[loc[pos]], min_s[pos], len[pos]);
                }
            } else if (ret == SWDOWN_PRESSED){
                if (val > min[pos]){
                    val--;
                    if (pos == 3){  // day of week
                        date1[loc[pos]]--;
                    } else {
                        if (date1[loc[pos] + 1] == '0'){
                            date1[loc[pos] + 1] = '9';
                            date1[loc[pos]]--;
                        } else {
                            date1[loc[pos] + 1]--;
                        }
                    }
                } else {
                    val = max[pos];
                    strncpy(&date1[loc[pos]], max_s[pos], len[pos]);
                }
            } else if (ret == SWPOFF_PRESSED){
                continue_flag = 0;
                break;
            }
        }
    }

    LCD_LPrintln(0, LCD_BLANK_LINE);
    LCD_LPrintln(1, LCD_BLANK_LINE);
    return ret;
}


#INT_TIMER0
void isr_timer1(){
    uint16_t n;

    n = Timer1;
    if (n) Timer1 = --n;
    n = Timer2;
    if (n) Timer2 = --n;

    if (CHATTERING_TMR > 0){
        CHATTERING_TMR++;
        if (CHATTERING_TMR == CHATTERING_ADJUST) CHATTERING_TMR = 0;
    } else {
        BOOLEAN    SwCurrent;

        // sw_play(PLAY button) supports long time press
        SwCurrent = sw_mode();
        if (SwmodeLast == SwCurrent){
            if (SwCurrent == 0){   // press continues
                SW_CONTINUE_COUNT++;
            } else {    // release continues
                SW_CONTINUE_COUNT = 0;
            }
        } else {    // press state change (press or release)
            CHATTERING_TMR = 1;     // chattering timer start
            if (SwCurrent == 1){    // released
                //EVENT_SWMODE = 1;   // press event happens
                EVENT_SWANY = EVENT_SWMODE;
            }
        }
        SwmodeLast = SwCurrent;

        if (EVENT_SWANY > 0){
        } else if (sw_playrec() == 0){
            //EVENT_SWPLAYREC = 1;
            EVENT_SWANY = EVENT_SWPLAYREC;
            CHATTERING_TMR = 1;     // chattering timer start
        }
#if 0
        else if (sw_mode() == 0){
            //EVENT_SWMODE = 1;
            EVENT_SWANY = EVENT_SWMODE;
            CHATTERING_TMR = 1;     // chattering timer start
        }
#endif
        else if (sw_next() == 0){
            //EVENT_SWNEXT = 1;
            EVENT_SWANY = EVENT_SWNEXT;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        else if (sw_prev() == 0){
            //EVENT_SWPREV = 1;
            EVENT_SWANY = EVENT_SWPREV;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        else if (sw_up() == 0){
            //EVENT_SWUP = 1;
            EVENT_SWANY = EVENT_SWUP;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        else if (sw_down() == 0){
            //EVENT_SWDOWN = 1;
            EVENT_SWANY = EVENT_SWDOWN;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        else if (sw_detectpoff() == 0){
            int8    ret = OnRTCAlarm();
            if (ret == 1){          // power on alarm
                //EVENT_RECPREP = 1;
                EVENT_SWANY = EVENT_RECPREP;
            } else if (ret == 2){   // recording start
                //EVENT_RECSTART = 1;
                EVENT_SWANY = EVENT_RECSTART;
            } else if (ret == 3){
                //EVENT_RECSTOP = 1;
                EVENT_SWANY = EVENT_RECSTOP;
            } else {
                //EVENT_SWPOFF = 1;
                EVENT_SWANY = EVENT_SWPOFF;
            }
            CHATTERING_TMR = 1;     // chattering timer start
        }
    }
    
    set_timer0(Timer0InitValue);
//    set_timer1(Timer1InitValue);
}

/*-----------------------------------------------------------------------*/
/* Wait for switch event                                                 */
/*-----------------------------------------------------------------------*/
int8 PauseAndSelectItem(){
    int8 ret;
    
loop_PauseAndSelectItem:
    if (EVENT_SWANY > 0 && CHATTERING_TMR == 0){
        switch (EVENT_SWANY){
        case EVENT_SWPLAYREC:
            ret = SWPLAYREC_PRESSED;
            break;
        case EVENT_SWMODE:
            ret = SWMODE_PRESSED;
            break;
        case EVENT_SWNEXT:
            ret = SWNEXT_PRESSED;
            break;
        case EVENT_SWPREV:
            ret = SWPREV_PRESSED;
            break;
        case EVENT_SWUP:
            ret = SWUP_PRESSED;
            break;
        case EVENT_SWDOWN:
            ret = SWDOWN_PRESSED;
            break;
        case EVENT_SWPOFF:
            ret = SWPOFF_PRESSED;
            break;
        case EVENT_RECPREP:
            ret = RECPREP_ALARMED;
            break;
        case EVENT_RECSTART:
            ret = RECSTART_ALARMED;
            break;
        case EVENT_RECSTOP:
            ret = RECSTOP_ALARMED;
            break;
        default:
            EVENT_SWANY = 0;
            goto loop_PauseAndSelectItem;
            break;
        }
        
        EVENT_SWANY = 0;
        return ret;
    }

    goto loop_PauseAndSelectItem;

    return FR_OK;
}

void WritePluginToEEPROM(char *PluginFile, FIL *file){
    int16       i = 0;
    FRESULT     res;
    uint16_t    adrs;
    UINT        br;
    uint8_t     buff[16];

    res = f_open(file, PluginFile, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        return;
    }
    adrs = 0;
    while (1){
        res = f_read(file, &buff, 16, &br);
        if (res != FR_OK || br == 0){
            break;
        }
        for (i=0; i<br; i++){
            while (AT24C128_Write(adrs, buff[i]) == 1) ;
            adrs++;
        }
    }
    f_close(file);
}

/*-----------------------------------------------------------------------*/
/* Check plugin data                                                     */
/*-----------------------------------------------------------------------*/
void CheckPluginInEEPROM(char *PluginFile, FIL *file){
    int16       i = 0;
    FRESULT     res;
    uint16_t    adrs;
    UINT        br;
    uint8_t     data;
    uint16_t    ok, ng;
    uint8_t     buff[8];

    while (AT24C128_SeqRead_Start(0) == 1) ;

    ok = 0;
    ng = 0;
    res = f_open(file, PluginFile, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        return;
    }
    adrs = 0;
    while (1){
        res = f_read(file, &buff, 8, &br);
        if (res != FR_OK || br == 0){
            break;
        }
        for (i=0; i<br; i++){
            AT24C128_SeqRead_Read(&data, 1);
            adrs++;
            if (data == buff[i]){
                ok++;
            } else {
                ng++;
            }
        }
    }
    f_close(file);
    AT24C128_SeqRead_Stop();

    if (ng > 0){
        char    buff[8];
        itoa(ng, 10, buff);
        LCD_LPrintln(1, "Punmatch        ");
        LCD_LCPrintln(1, 9, buff);
        wait_ms(5000);
    }
}

/*-----------------------------------------------------------------------*/
/* Check plugin data                                                     */
/*-----------------------------------------------------------------------*/
void CheckPluginInEEPROM2(char *PluginFile, FIL *file){
    FRESULT     res;
    uint16_t    memad;
    UINT        br;
    uint16_t    data1, data2;
    uint16_t    ok, ng;

    memad = 0;
    ok = 0;
    ng = 0;
    res = f_open(file, PluginFile, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        return;
    }
    while (1){
        res = f_read(file, &data1, 1, &br);
        if (res != FR_OK || br != 1){
            break;
        }
        while (AT24C128_Read(memad, &data2) == 1) ;
        memad++;
        if (data1 == data2){
            ok++;
        } else {
            ng++;
        }
    }
    f_close(file);

    if (ng > 0){
        LCD_LPrintln(1, "Plugin unmatch  ");
    }
}

/*-----------------------------------------------------------------------*/
/* Load Plugin to VS1063                                                 */
/*-----------------------------------------------------------------------*/
void LoadPluginFromEEPROM(){
    int16       i = 0;
    uint16_t    num;

    while (AT24C128_SeqRead_Start(0) == 1) ;
    AT24C128_SeqRead_Read(&num, 2);

    while (i<num) {
        uint16_t addr, n, val;

        AT24C128_SeqRead_Read(&addr, 2);
        i++;
        AT24C128_SeqRead_Read(&n, 2);
        i++;
        if (n & 0x8000U) { /* RLE run, replicate n samples */
            n &= 0x7FFF;
            AT24C128_SeqRead_Read(&val, 2);
            i++;
            while (n--) {
                while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        } else {           /* Copy run, copy n samples */
            while (n--) {
                AT24C128_SeqRead_Read(&val, 2);
                i++;
                while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        }
    }

    AT24C128_SeqRead_Stop();
}

/*-----------------------------------------------------------------------*/
/* Load Plugin to VS1063                                                 */
/*-----------------------------------------------------------------------*/
void LoadPluginFromFile(char *PluginFile, FIL *file) {
    int16       i = 0;
    FRESULT     res;
    uint16_t    num;
    UINT        br;

    // get number of data
    res = f_open(file, PluginFile, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        //write_eeprom(EP_ADRS_LP_0, res);
    }
    res = f_read(file, &num, 2, &br);
    if (res != FR_OK || br != 2){
        //write_eeprom(EP_ADRS_LP_1, res);
    }

    while (i<num) {
        //  while (i<sizeof(plugin)/sizeof(plugin[0])) {
        uint16_t addr, n, val;
        //    unsigned short addr, n, val;
        //addr = plugin[i++];
        //n = plugin[i++];
        res = f_read(file, &addr, 2, &br);
        if (res != FR_OK){
            //write_eeprom(EP_ADRS_LP_2, res);
        }
        i++;
        res = f_read(file, &n, 2, &br);
        if (res != FR_OK){
            //write_eeprom(EP_ADRS_LP_3, res);
        }
        i++;
        if (n & 0x8000U) { /* RLE run, replicate n samples */
            n &= 0x7FFF;
            //val = plugin[i++];
            res = f_read(file, &val, 2, &br);
            if (res != FR_OK){
                //write_eeprom(EP_ADRS_LP_4, res);
            }
            i++;
            while (n--) {
                while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        } else {           /* Copy run, copy n samples */
            while (n--) {
                //val = plugin[i++];
                res = f_read(file, &val, 2, &br);
                if (res != FR_OK){
                    //write_eeprom(EP_ADRS_LP_5, res);
                }
                i++;
                while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        }
    }

    f_close(file);

}

/*-----------------------------------------------------------------------*/
/* New MP3 file name                                                     */
/*  YYMMDDnn.MAP (nn:sequencial number)                                  */
/*-----------------------------------------------------------------------*/
int8 GetNextMp3FileName(char *Mp3File, FIL *file){
    char    filename[16];
    int8    i;
    BOOLEAN flg;
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    UINT        br;
    uint16_t    num;
    int8        ret;
    
    ret = 0;
    flg = 0;
    RTC_GetCurrentDateYYMMDD(filename);

    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
        if (res == FR_OK && br == RECLEN_RECORDEDFILE){
            num = *((uint16_t *)&buff[0]);      // Num of recorded files
            if (num >= 1){
                res = f_lseek(file, (DWORD)num * RECLEN_RECORDEDFILE);  // last record
                res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
                flg = 0;
                for (i=0; i<6; i++){
                    if (filename[i] != buff[i]){
                        flg = 1;
                        break;
                    }
                }
                if (flg == 0){
                //if (strncmp(filename, buff, 6) == 0){       // matches year-month-day
                    if (buff[7] == '9'){
                        if (buff[6] == '9'){
                            // no more fileno
                            ret = 1;
                        } else {
                            filename[6] = buff[6] + 1;
                            filename[7] = '0';
                        }
                    } else {
                        filename[6] = buff[6];
                        filename[7] = buff[7] + 1;
                    }
                    filename[8] = '\0';
                    strcat(filename, ".MP3");
                } else {
                    strcat(filename, "01.MP3");
                }
            } else {
                strcat(filename, "01.MP3");
            }
        }
        
        f_close(file);
    } else {
        ret = 1;
    }
    
    strcpy(Mp3File, filename);
    
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Write new recorded file to list                                       */
/* Returns new added record no (1- )                                     */
/*-----------------------------------------------------------------------*/
uint16_t AddRecordedFileToList(FIL *file, char *Mp3File, uint8_t *recstartdt, char *stname, uint16_t freq){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    uint16_t    num;
    UINT        br;
    uint8_t     i;

    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_WRITE | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
        if (res == FR_OK && br == RECLEN_RECORDEDFILE){
            num = *((uint16_t *)&buff[0]);      // Num of recorded files
        } else {
            num = 0;
        }
        num++;
        // update header record
        *((uint16_t *)&buff[0]) = num;
        res = f_lseek(file, 0);
        res = f_write(file, buff, RECLEN_RECORDEDFILE, &br);
        // add new file record
        res = f_lseek(file, (DWORD)num * RECLEN_RECORDEDFILE);
        memset(buff, 0, RECLEN_RECORDEDFILE);
        strncpy(buff, Mp3File, MP3FILENAME_BUF_LEN);
        for (i=0; i<6; i++){
            buff[RF_RECSTARTDT + i] = recstartdt[i];
        }
        strncpy(buff + RF_STATIONNAME, stname, STATIONNAME_BUF_LEN);
        memcpy(&buff[RF_FREQUENCY], &freq, 2);
        res = f_write(file, buff, RECLEN_RECORDEDFILE, &br);
        f_close(file);
        
        return num;     // record no.
    } else {
        return 0;
    }
}

void SaveMp3Filesize(FIL *file, uint16_t recno, uint32_t filesize){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    UINT        br;

    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_WRITE | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_lseek(file, (DWORD)recno * RECLEN_RECORDEDFILE);
        res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
        memcpy(&buff[RF_FILESIZE], &filesize, 4);
        res = f_lseek(file, (DWORD)recno * RECLEN_RECORDEDFILE);
        res = f_write(file, buff, RECLEN_RECORDEDFILE, &br);
        f_close(file);
    }
}

void SaveRecordingStopDatetime(FIL *file, uint16_t recno, uint8_t *recstopdt){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    uint8_t     recstartdt[6];
    UINT        br;
    uint8_t     i;
    uint32_t    startdt, stopdt;
   
    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_WRITE | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_lseek(file, (DWORD)recno * RECLEN_RECORDEDFILE);
        res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
        for (i=0; i<6; i++){
            recstartdt[i] = buff[RF_RECSTARTDT + i];
            buff[RF_RECSTOPDT + i] = recstopdt[i];
        }
        //
        startdt = (uint32_t)Bcd2Int(recstartdt[5]) + 60UL*(uint32_t)Bcd2Int(recstartdt[4]) + 60UL*60UL*(uint32_t)Bcd2Int(recstartdt[3]);
        stopdt = (uint32_t)Bcd2Int(recstopdt[5]) + 60UL*(uint32_t)Bcd2Int(recstopdt[4]) + 60UL*60UL*(uint32_t)Bcd2Int(recstopdt[3]);
        if (recstartdt[2] != recstopdt[2]){     // don't thing over-24H recording time
            stopdt += 60UL*60UL*24UL;
        }
        stopdt -= startdt;
        memcpy(&buff[RF_TOTALSEC], &stopdt, 4);
        //
        res = f_lseek(file, (DWORD)recno * RECLEN_RECORDEDFILE);
        res = f_write(file, buff, RECLEN_RECORDEDFILE, &br);
        f_close(file);
    }
}


/*-----------------------------------------------------------------------*/
/* Create a filenamelist                                                 */
/*-----------------------------------------------------------------------*/
void CreateRecordedFileList(FIL *file){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    UINT        br, i;

    res = f_open(file, RECORDEDFILELIST, FA_WRITE | FA_CREATE_NEW);
    if (res == FR_OK){
        memset(buff, 0, RECLEN_RECORDEDFILE);
        for (i=0; i<1000; i++){
            f_write(file, buff, RECLEN_RECORDEDFILE, &br);
        }
        f_close(file);
    }
}

/*
 *  fileno : 1 - num of recorded files
 */
int GetRecordedFileName(FIL *file, uint16_t fileno, char *Mp3File){
    FRESULT     res;
    char        buff[RECLEN_RECORDEDFILE];
    UINT        br;

    res = f_open(file, RECORDEDFILELIST, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_lseek(file, (DWORD)fileno * RECLEN_RECORDEDFILE);
        if (res == FR_OK){
            res = f_read(file, buff, RECLEN_RECORDEDFILE, &br);
            if (res == FR_OK && br == RECLEN_RECORDEDFILE){
                memcpy(Mp3File, buff, 13);
                f_close(file);
                return FR_OK;
            }
        }
        f_close(file);
        return FR_NO_FILE;
    }
    return FR_NO_FILE;
}


/*-----------------------------------------------------------------------*/
/* Set Station List On EEPROM by Station list file                       */
/*-----------------------------------------------------------------------*/
int8 SetStationList(FIL *file){
    FRESULT     res;
    char        buff[48];
    char        *c;
    uint16_t    freq;
    uint8_t     fm_n;
    uint8_t     am_n;
    uint16_t    adrs;
    int8        i;

    fm_n = 0;
    am_n = 0;
    res = f_open(file, STATIONLISTFILE, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        while (f_gets(buff, 32, file) != NULL){
            if ((buff[0] == 'A' || buff[0] == 'F') && buff[1] == ':'){
                c = strchr(&buff[2], ':');
                if (c != NULL){
                    *c = '\0';
                    freq = atol(&buff[2]);
                    c++;        // Station name

                    if (buff[0] == 'A'){
                        if (am_n >= NUM_MAX_STATIONS) continue;
                        am_n++;
                        adrs = EEPROM_AM_START + 0x010*(am_n - 1);
                    } else if (buff[0] == 'F'){
                        if (fm_n >= NUM_MAX_STATIONS) continue;
                        fm_n++;
                        adrs = EEPROM_FM_START + 0x010*(fm_n - 1);
                    }
                    // frequency
                    write_eeprom(adrs , freq & 0x00FF);
                    write_eeprom(adrs + 1 , (freq & 0xFF00) >> 8);
                    // station name
                    for (i=0; i<12; i++){
                        if (*c == '\0' || *c == '\r' || *c == '\n') break;
                        write_eeprom(adrs + 2 + i , *c++);
                    }
                    write_eeprom(adrs + 2 + i , '\0');  // null terminate
                }
            }
        }
        f_close(file);

        // num of stations
        write_eeprom(EEPROM_FM_STATIONS , fm_n);
        write_eeprom(EEPROM_AM_STATIONS , am_n);
    }

    return 0;
}

uint8_t GetFrequencyListIndex(uint8_t fmam, uint16_t freq){
    uint16_t    adrs;
    uint8_t     i;
    uint16_t    f;
    uint8_t     NumStations;
    
    if (fmam == RADIO_BAND_FM){     // FM
        adrs = EEPROM_FM_START;
        NumStations = read_eeprom(EEPROM_FM_STATIONS);
    } else {            // AM
        adrs = EEPROM_AM_START;
        NumStations = read_eeprom(EEPROM_AM_STATIONS);
    }
    for (i=0; i<NumStations; i++){
        f = read_eeprom(adrs + 1);
        f <<= 8;
        f |= read_eeprom(adrs);
        if (f == freq) return i;
        
        adrs += 0x010;
    }
    return 0xFF;
}

/*-----------------------------------------------------------------------*/
/* Get frequency&name from Station List On EEPROM                        */
/*-----------------------------------------------------------------------*/
int8 GetFrequencyAndStationName(uint8_t fmam, uint8_t index, uint16_t *freq, char stname[]){
    uint16_t    adrs;
    uint8_t     i;

    if (!(index >= 0 && index < NUM_MAX_STATIONS)) return 1;

    if (fmam == RADIO_BAND_FM){     // FM
        adrs = EEPROM_FM_START + 0x010*index;
    } else {            // AM
        adrs = EEPROM_AM_START + 0x010*index;
    }

    // frequency
    *freq = read_eeprom(adrs + 1);
    *freq <<= 8;
    *freq |= read_eeprom(adrs);
    // name
    for (i=0; i<STATIONNAME_BUF_LEN; i++){
        stname[i] = read_eeprom(adrs + 2 + i);
    }
    //WriteLog("GetFrequencyAndStationName", 0);
    //WriteLog(stname, 0);
    
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Get next recording schedule from file                                 */
/* And set this to EEPROM                                                */
/*-----------------------------------------------------------------------*/
int8 SetNextScheduleToEEPROM(FIL *file){
    FRESULT     res;
    char        buff1[64];
    char        buff2[16];
    char        *c;
    uint8_t     pondatetime[6];
    uint8_t     startdatetime[6];
    uint8_t     stopdatetime[6];
    char        amfm;
    uint16_t    freq;
    uint8_t     fidx;
    char        sp;
    uint8_t     ec;
    uint8_t     curdt[8];
    int8        i;
    BOOLEAN     newschflg;
    BOOLEAN     errflg;
    int8        lineno;
    static uint8_t  lastday[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    
    RTC_GetCurrentDateTimeBCD(curdt);
    
    res = f_open(file, SCHEDULEFILE, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        WriteLog(" Schedule file check ", 1);

        newschflg = 0;
        errflg = 0;
        lineno = 0;
        while (f_gets(buff1, 64, file) != NULL){
            if (buff1[0] == '#') continue;
            
            WriteLog(buff1, 0);
            lineno++;
            
            if (buff1[6] == ':' && buff1[13] == ':' && buff1[20] == ':' && buff1[27] == ':'){
                // convert recording start datetime to BCD ("YYMMDD:hhmmss" -> "YYMMDDhhmmss")
                strncpy(&buff2[0], &buff1[0], 6);
                strncpy(&buff2[6], &buff1[7], 6);
                buff2[12] = '\0';
                if (ConvNumericStrToBCD(buff2, startdatetime) == 1){
                    WriteLog("Cannot convert start date to BCD", 0);
                    errflg = 1;
                    continue;
                }
                
                // Copy recording start datetime to pondatetime
                for (i=0; i<6; i++){
                    pondatetime[i] = startdatetime[i];
                }
                // think leap year
                i = Bcd2Int(pondatetime[0]);
                if (i == 16 || i == 20 || i == 24 || i == 28 || i == 32) lastday[2] = 29;
                else lastday[2] = 28;
                
                // Get power on time (1 min before recording start)
                i = Bcd2Int(pondatetime[4]);        // min
                if (i > 0){
                    i--;
                    pondatetime[4] = Int2Bcd(i);
                } else {
                    pondatetime[4] = 0x59;
                    i = Bcd2Int(pondatetime[3]);    // hour
                    if (i > 0){
                        i--;
                        pondatetime[3] = Int2Bcd(i);
                    } else {
                        pondatetime[3] = 0x23;
                        i = Bcd2Int(pondatetime[2]);        // day
                        if (i > 1){
                            i--;
                            pondatetime[2] = Int2Bcd(i);
                        } else {
                            i = Bcd2Int(pondatetime[1]);    // month
                            pondatetime[2] = Int2Bcd(i == 1 ? lastday[12] : lastday[i - 1]);    // last day of previous month
                            if (i > 1){
                                i--;
                                pondatetime[1] = Int2Bcd(i);
                            } else {
                                pondatetime[1] = 0x12;
                                i = Bcd2Int(pondatetime[0]);    // year
                                i--;
                                pondatetime[0] = Int2Bcd(i);
                            }
                        }
                    }
                }
                // Recording stop datetime to BCD
                strncpy(&buff2[0], &buff1[14], 6);
                strncpy(&buff2[6], &buff1[21], 6);
                buff2[12] = '\0';
                if (ConvNumericStrToBCD(buff2, stopdatetime) == 1){
                    WriteLog("Cannot convert stop date to BCD", 0);
                    errflg = 1;
                    continue;
                }
            } else {
                WriteLog("Invalid Schedule DateTime separator", 0);
                errflg = 1;
                continue;
            }
            // AM/FM
            amfm = buff1[28];
            if (amfm == 'F'){
                amfm = RADIO_BAND_FM;
            } else if (amfm == 'A'){
                amfm = RADIO_BAND_AM;
            } else {
                WriteLog("Invalid Schedule FM/AM", 0);
                errflg = 1;
                continue;
            }
            // frequency
            c = strchr(&buff1[30], ':');
            if (c != NULL){
                *c = '\0';
                freq = atol(&buff1[30]);
                fidx = GetFrequencyListIndex(amfm, freq);
                if (fidx == 0xFF){
                    WriteLog("Invalid Schedule Freq", 0);
                    errflg = 1;
                    continue;
                }
            } else {
                WriteLog("Invalid Schedule Freq", 0);
                errflg = 1;
                continue;
            }
            c++;
            // SP
            if (*c == SCHEDULED_REC_SPEAKER_ON || *c == SCHEDULED_REC_SPEAKER_OFF){
                sp = *c;
            } else {
                WriteLog("Invalid Schedule Speaker", 0);
                errflg = 1;
                continue;;
            }
            c += 2;
            // Encode Setting
            ec = (*c - '0');
            if (ec >= 0 && ec < NUMENCODESETTINGS){         // NUMENCODESETTINGS : vs1063.c
            } else {
                WriteLog("Invalid Schedule Encode Setting", 0);
                errflg = 1;
                continue;
            }
            
            // compare start datetime with current datetime
            if (CompareDateBCD(startdatetime, curdt) < 0){
                LCD_LPrintln(1, "New Schedule    ");
                
                if (newschflg == 0){
                    WriteLog("Set to EEPROM", 0);
                    // save to EEPROM
                    for (i=0; i<6; i++){
                        write_eeprom(EEPROM_REC_SCHEDULE1 + i, pondatetime[i]);
                    }
                    for (i=0; i<6; i++){
                        write_eeprom(EEPROM_REC_SCHEDULE2 + i, startdatetime[i]);
                    }
                    write_eeprom(EEPROM_REC_SCHEDULE2_FMAM, amfm);
                    write_eeprom(EEPROM_REC_SCHEDULE2_FREQ_LB, freq & 0x00FF);
                    write_eeprom(EEPROM_REC_SCHEDULE2_FREQ_UB, (freq >> 8) & 0x00FF);
                    write_eeprom(EEPROM_REC_SCHEDULE2_FINDEX, fidx);
                    write_eeprom(EEPROM_REC_SCHEDULE2_SPEAKER, sp);
                    write_eeprom(EEPROM_REC_SCHEDULE2_ENCSET, ec);
                    for (i=0; i<6; i++){
                        write_eeprom(EEPROM_REC_SCHEDULE3 + i, stopdatetime[i]);
                    }
                }
                newschflg = 1;
            } else {
                WriteLog("Old Schedule", 0);
            }
        }
        f_close(file);
        
        if (errflg == 1){
            LCD_LPrintln(1, "Error in schedule");
            wait_ms(5000);
        }
        if (newschflg == 0){  // No recording schedule
            for (i=0; i<6; i++){
                write_eeprom(EEPROM_REC_SCHEDULE1 + i, 0xFF);
                write_eeprom(EEPROM_REC_SCHEDULE2 + i, 0xFF);
                write_eeprom(EEPROM_REC_SCHEDULE3 + i, 0xFF);
            }
        }
    }

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Set RTC alarm for recording                                           */
/*  alarmno : 1=Power On / 2=Recording start / 3=Recording stop          */
/*-----------------------------------------------------------------------*/
int8 SetRecordingAlarmToRTC(uint8_t alarmno){
    int8    i;
    uint8_t     datetime[6];
    
    for (i=0; i<6; i++){
        if (alarmno == 1) datetime[i] = read_eeprom(EEPROM_REC_SCHEDULE1 + i);     // power on time
        else if (alarmno == 2) datetime[i] = read_eeprom(EEPROM_REC_SCHEDULE2 + i);
        else if (alarmno == 3) datetime[i] = read_eeprom(EEPROM_REC_SCHEDULE3 + i);
    }
    if (!(datetime[0] >= 0x15 && datetime[0] <= 0x35)){     // check year range
        RTC_ClearAlarm();
        return 1;
    }
    
    RTC_SetAlarm(datetime);
    write_eeprom(EEPROM_ALARM_NO, alarmno);
    
    return 0;
}


/*-----------------------------------------------------------------------*/
/*  Check RTC power on                                                   */
/*      AF bit is On.                                                    */
/*      EEPROM_ALARM_NO is 1.                                            */
/*      Current time is between Power On time and recording start time.  */
/*-----------------------------------------------------------------------*/
BOOLEAN IsRtcPowerOn(){
    uint8_t     curtime[8];
    uint8_t     schtime[8];
    int8        i;
    
    if (OnRTCAlarm() == 1){
        write_eeprom(EEPROM_DEBUG_1 + 3, 1);
    }
    
    RTC_GetCurrentDateTimeBCD(curtime);

    write_eeprom(EEPROM_DEBUG_1, 1);
    // RTC Power On alarm time 
    for (i=0; i<6; i++){
        schtime[i] = read_eeprom(EEPROM_REC_SCHEDULE1 + i);
    }
    // compare : current time must be after power on time
    if (CompareDateBCD(schtime, curtime) < 0){
        return 0;
    }
    write_eeprom(EEPROM_DEBUG_1 + 1, 1);

    // RTC Encode start alarm time 
    for (i=0; i<6; i++){
        schtime[i] = read_eeprom(EEPROM_REC_SCHEDULE2 + i);
    }
    // compare : current time must be before recording start time
    if (CompareDateBCD(schtime, curtime) > 0){
        return 0;
    }
    write_eeprom(EEPROM_DEBUG_1 + 2, 1);

    return 1;

    //return 0;
}


/*-----------------------------------------------------------------------*/
/*  Return the AlarmNo of current alarm                                  */
/*      1= power on time                                                 */
/*      2= start recording time                                          */
/*      3= stop recording time                                           */
/*-----------------------------------------------------------------------*/
int8 OnRTCAlarm(){
    uint8_t     curtime[16];
    uint8_t     alarmdt[6];
    int8    i;
    uint8_t flags;
    uint8_t alarmno;
    
    flags = RTC_GetFlags();     // for reset IRQ output
    if ((flags & 0x40) != 0){   // AF bit
        alarmno = read_eeprom(EEPROM_ALARM_NO);
        if ((flags & 0x04) != 0){   // OF bit
            RTC_ResetFlags();
        }
        RTC_ClearAlarm();
        
        // debug info
        for (i=0; i<6; i++){
            if (alarmno == 1) alarmdt[i] = read_eeprom(EEPROM_REC_SCHEDULE1 + i);     // power on time
            else if (alarmno == 2) alarmdt[i] = read_eeprom(EEPROM_REC_SCHEDULE2 + i);
            else if (alarmno == 3) alarmdt[i] = read_eeprom(EEPROM_REC_SCHEDULE3 + i);
            else alarmdt[i] = 0;
            //write_eeprom(EEPROM_DEBUG_1 + i, alarmdt[i]);
        }
        RTC_GetCurrentDateTimeBCD(curtime);
        for (i=0; i<6; i++){
            //write_eeprom(EEPROM_DEBUG_1 + 6 + i, curtime[i]);
        }
        //write_eeprom(EEPROM_DEBUG_1 + 12, RTC_GetFlags());      // Confirm AF/OF flags are 0
        //write_eeprom(EEPROM_DEBUG_1 + 13, alarmno);      // Confirm AF/OF flags are 0
        
        return alarmno;
    }
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Get current recording schedule from EEPROM                            */
/*-----------------------------------------------------------------------*/
int8 GetCurrentScheduleFromEEPROM(RECCONDITION *recconf){
    uint16_t    freq;
     
    freq = read_eeprom(EEPROM_REC_SCHEDULE2_FREQ_UB);
    freq <<= 8;
    freq |= read_eeprom(EEPROM_REC_SCHEDULE2_FREQ_LB);
    if (freq == 0 || freq == 0xFFFF) return 1;        // No schedule
    
    recconf->FmAm = read_eeprom(EEPROM_REC_SCHEDULE2_FMAM);
    recconf->Freq = freq;
    recconf->Index = read_eeprom(EEPROM_REC_SCHEDULE2_FINDEX);
    recconf->SpeakerOn = read_eeprom(EEPROM_REC_SCHEDULE2_SPEAKER);
    recconf->EncSet = read_eeprom(EEPROM_REC_SCHEDULE2_ENCSET);
    strcpy(recconf->StName, "SCHEDULED REC");
    
#if 0
    // freq is in station list ?
    for (i=0; i<NUM_MAX_STATIONS; i++){
        if (fmam == 0){     // FM
            adrs = EEPROM_FM_START + 0x010*i;
        } else {            // AM
            adrs = EEPROM_AM_START + 0x010*i;
        }
        freqw = read_eeprom(adrs + 1);
        freqw <<= 8;
        freqw |= read_eeprom(adrs);
        
        if (freqw == freq){
            // station name
            for (j=0; j<STATIONNAME_BUF_LEN; j++){
                recconf->StName[j] = read_eeprom(adrs + 2 + j);
                recconf->Index = i;
            }
            break;
        }
    }
    if (i == NUM_MAX_STATIONS){     // do not exist in station list
        strcpy(recconf->StName, "             ");
        recconf->Index = 0xFF;
    }
#endif
    
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Sub Mode                                                              */
/*-----------------------------------------------------------------------*/
int8 SubMode(){
    int8                ret;

    ret = SelectSubMode();
    if (ret == SUBMODE_ADJUST_RTC){
        ret = AdjustRtc();
    } else if (ret == SUBMODE_ENCODE_SETTING){
        ret = SelectEncodeSetting();
    } else if (ret == SUBMODE_CONFIRM_SCHEDULE){
        ret = ConfirmRecordingSchedule();
    } else if (ret == SUBMODE_RETURN){
        ret = 0;
    }
    
    return ret;
}

int8 SelectSubMode(){
    SUBMODE    submode;
    int8                ret;

    submode = 0;
    
    LCD_LPrintln(0, "SELECT SUB-MODE:");
    while(1){
        switch (submode){
            case SUBMODE_ADJUST_RTC:
                LCD_LPrintln(1, "1:Adjust RTC    ");
                break;
            case SUBMODE_ENCODE_SETTING:
                LCD_LPrintln(1, "2:Encode Setting");
                break;
            case SUBMODE_CONFIRM_SCHEDULE:
                LCD_LPrintln(1, "3:Confirm Schdul");
                break;
            case SUBMODE_RETURN:
                LCD_LPrintln(1, "4:End Sub-Menu  ");
                break;
        }
        ret = PauseAndSelectItem();
        if (ret == SWMODE_PRESSED || ret == SWNEXT_PRESSED){
            submode++;
            if (submode == SUBMODE_COUNT){
                submode = 0;
            }
        } else if (ret == SWPLAYREC_PRESSED){
            return submode;
        }
    }
}
/*-----------------------------------------------------------------------*/
/* Convert null terminated numeric string to BCD                         */
/*-----------------------------------------------------------------------*/
int8 ConvNumericStrToBCD(char *numstr, uint8_t *bcd){
    uint8_t     i;
    
    for (i=0; ; i+=2){
        if (numstr[i] == '\0') break;
        if (!(numstr[i] >= '0' && numstr[i] <= '9')) return 1;
        if (!(numstr[i+1] >= '0' && numstr[i+1] <= '9')) return 1;
        
        bcd[i/2] = ((unsigned char)(numstr[i] - '0') << 4 | (numstr[i + 1] - '0'));
    }
    return 0;
}

/*-----------------------------------------------------------------------*/
/*  for FatFs                                                            */
/*-----------------------------------------------------------------------*/
DWORD get_fattime (void)
{
    uint8_t     dt[6];
    
    RTC_GetCurrentDateTimeBCD(dt);
    
	return	  ((DWORD)(Bcd2Int(dt[0]) + (2000 - 1980)) << 25)	/* Y */
			| ((DWORD)Bcd2Int(dt[1])  << 21)			/* M */
			| ((DWORD)Bcd2Int(dt[2]) << 16)				/* D */
			| ((DWORD)Bcd2Int(dt[3])  << 11)			/* H */
			| ((DWORD)Bcd2Int(dt[4])  << 5)				/* M */
			| ((DWORD)Bcd2Int(dt[5])  >> 1);			/* S */
}


/*-----------------------------------------------------------------------*/
/*  date1 < date2 : return +1                                   */
/*-----------------------------------------------------------------------*/
int8 CompareDateBCD(uint8_t *date1, uint8_t *date2){
    int8    i;
    
    for (i=0; i<6; i++){
        if (date1[i] > date2[i]) return -1;
        else if (date1[i] < date2[i]) return +1;
    }
    return 0;
}

void ConvBCDtoStr(uint8_t *bcd, uint8_t len, char *buf){
    uint8_t     i;
    
    for (i=0; i<len; i++){
        buf[2*i] = itoc(bcd[i] >> 4);
        buf[2*i + 1] = itoc(bcd[i] & 0x0F);
    }
    buf[2*len] = '\0';
}


uint16_t Int2Bcd(uint16_t i){
    return (i % 10) | ((i / 10) << 4);
}

uint16_t Bcd2Int(uint16_t b){
    return (b & 0x0F) + ((b & 0xF0) >> 4) * 10;
}

/*-----------------------------------------------------------------------*/
/*  Wait millisecond                                                     */
/*-----------------------------------------------------------------------*/
void wait_ms(uint16_t ms){
    uint8_t     i;
    uint16_t    m;
    
    for (m=0; m<ms; m++){
        
        // 1ms
        for (i=0; i<WaitMsCycle; i++){
            delay_cycles(250);
        }
    }
}


#if 0
#INT_TIMER0
void isr_timer0(){
    Timer0Count++;
    if (Timer0Count == 5){
        EVENT_TIMER0 = 1;
        Timer0Count = 0;
    }
    set_timer0(Timer0InitValue);
}
#endif

void WriteLog(char *buff, int dtout){
    UINT    bw;
    char    cdate[16];
    
    if (dtout == 1){
        RTC_GetCurrentDateTimeForDisp(cdate);
        f_write(&FileLog, cdate, strlen(cdate), &bw);
    }
    f_write(&FileLog, buff, strlen(buff), &bw);
    f_write(&FileLog, "\r\n", 2, &bw);
    f_sync(&FileLog);
}
