/* 
 * File:   at24c128.c
 * 
 * Created on 2015/10/13
 *
 * EEPROM ATMEL AT24C128
 */

#define MEM_ADDRESS         0xA0           // 1010 000 0


/*-----------------------------------------------------------------------*/
/* Write to AT24C128                                                     */
/*-----------------------------------------------------------------------*/
uint8_t AT24C128_Write(uint16_t address, uint8_t data){
    int8    ack;
    uint8_t ret = 0;
    uint8_t devad;
    uint8_t i;

    devad = MEM_ADDRESS;

    i = 10;
    i2c_start();
    do {
        ack = i2c_write(devad);
    } while (ack != 0 && i-- > 0);
    //write_eeprom(0x00, ack);		// for debug
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    delay_ms(10);
    ack = i2c_write(address >> 8);
    //write_eeprom(0x01, ack);
    if (ack == 1){
        ret = 2;
        goto stop_condition;
    }
    delay_ms(10);
    ack = i2c_write(address & 0xFF);
    //write_eeprom(0x02, ack);
    if (ack == 1){
        ret = 3;
        goto stop_condition;
    }
    delay_ms(10);
    ack = i2c_write(data);
    //write_eeprom(0x03, ack);
    if (ack == 1){
        ret = 4;
        goto stop_condition;
    }
    delay_ms(10);

stop_condition:

    i2c_stop();
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Read from AT24C128                                                    */
/*-----------------------------------------------------------------------*/
uint8_t AT24C128_Read(uint16_t address, uint8_t *data){
    int8    ack;
    //uint8_t data;
    uint8_t    ret = 0;
    uint8_t     devad;

    devad = MEM_ADDRESS;
    
    i2c_start();
    // Device Address + Write Bit
    ack = i2c_write(devad);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(1);
    //write_eeprom(0x30, ack);
    // 1st WORD Address
    ack = i2c_write(address >> 8);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(1);
    //write_eeprom(0x31, ack);
    // 2nd WORD Address
    ack = i2c_write(address & 0xFF);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(1);
    //write_eeprom(0x32, ack);
    i2c_start();
    // Device Address + Read Bit
    ack = i2c_write(devad | 0x01);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(1);
    //write_eeprom(0x33, ack);
    *data = i2c_read(0);
//    delay_ms(1);
    //write_eeprom(0x34, ack);

stop_condition:
    i2c_stop();
    return ret;
}

uint8_t AT24C128_SeqRead_Start(uint16_t address){
    int8        ack;
    uint8_t    ret = 0;

    i2c_start();
    // Device Address + Write Bit
    ack = i2c_write(MEM_ADDRESS);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    // 1st WORD Address
    ack = i2c_write(address >> 8);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    // 2nd WORD Address
    ack = i2c_write(address & 0xFF);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    i2c_start();
    // Device Address + Read Bit
    ack = i2c_write(MEM_ADDRESS | 0x01);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }

stop_condition:
    if (ret == 1) i2c_stop();
    return ret;
}

uint8_t AT24C128_SeqRead_Read(uint8_t *data, uint16_t readbytes){
    uint16_t i;

    for (i=0; i<readbytes; i++){
        *(data + i) = i2c_read(1);
    }

    return 0;
}

uint8_t AT24C128_SeqRead_Stop(){
    i2c_stop();
    return 0;
}

uint8_t AT24C128_Read_Word(uint16_t address, uint16_t *data){
    if (AT24C128_SeqRead_Start(address) == 0){
        AT24C128_SeqRead_Read(data, 2);
        AT24C128_SeqRead_Stop();
        return 0;
    } else {
        return 1;
    }
}

/*-----------------------------------------------------------------------*/
/* Test AT24C128                                                         */
/*-----------------------------------------------------------------------*/
uint8_t AT24C128_Test(){
    uint8_t devad, addrs, cnt1, cnt2, data;
    char    buff1[17], buff2[8];
    uint8_t ret;
    
    //write_eeprom(0x44, 0x10);

    devad = MEM_ADDRESS;
    for (addrs = 0; addrs <0x10; addrs++){
        if ((ret = AT24C128_Write(addrs, addrs & 0xFF)) != 0){
            itoa(ret, 10, buff2);
            strcpy(buff1, "WR ERR=");
            strcat(buff1, buff2);
            strcat(buff1, "  ");
            LCD_LPrintln(1, buff1);
            wait_ms(10000);
            return 1;
        }
        delay_ms(10);
    }
    cnt1 = cnt2 = 0;

    for (addrs = 0; addrs <0x10; addrs++){
        while (AT24C128_Read(addrs, &data) == 1) ;
        if (data == (addrs & 0xFF)){
            cnt1++;
        } else {
            cnt2++;
        }
        delay_ms(10);
        //if (addrs < 0x10) write_eeprom(addrs + 0x10, data);
        //delay_ms(10);
    }
    //write_eeprom(0x40, cnt1);
    //write_eeprom(0x41, cnt1 >> 8);
    //write_eeprom(0x42, cnt2);
    //write_eeprom(0x43, cnt2 >> 8);
    itoa(cnt1, 10, buff2);
    strcpy(buff1, "OK=");
    strcat(buff1, buff2);
    strcat(buff1, " NG=");
    itoa(cnt2, 10, buff2);
    strcat(buff1, buff2);
    strcat(buff1, "  ");
    LCD_LPrintln(1, buff1);
    wait_ms(10000);
    
    return 0;
}

