/*
 * File:   Lcd_I2C.c
 * Mp3_05
 *
 * Created on 2015/10/13
 */

#define LCD_ADDRESS         0x7C            // 0b0111 110 0
#define LCD_CONTRAST        0b110000
#define LCD_BLANK_LINE      "                "


uint8_t LCD_I2C_Start(){
    int8    ack;
    
    i2c_start();
    ack = i2c_write(LCD_ADDRESS);
    if (ack == 1){
        return 1;
    } else {
        return 0;
    }
}

uint8_t LCD_Init(){
    int8    ack;
    uint8_t    ret = 0;
    
//    output_high(LCD_RST);
//    output_high(PIN_LCD_I2C_RST);
// comment out 12/19    delay_ms(100);
    
    i2c_start();
    ack = i2c_write(LCD_ADDRESS);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x20, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b00111000);        // function set(DL=1:8bit interface date, N=1:2lines)
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x21, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b00111001);        // function set(DL=1:8bit interface date, N=1:2lines, IS=1:extension instruction)
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x22, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b00010100);        // interval osc(BS=0:Bias 1/5, F210=100:Internal osc freq. adjust)
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x23, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b01110000 | (LCD_CONTRAST & 0xF));        // contrast low
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x24, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b01011100 | ((LCD_CONTRAST >> 4) & 0x3));  // contrast high,
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x25, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b01101100);        // follower control
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x26, ack);

    //delay_ms(200);
    wait_ms(200);

    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b00111000);        // function set(DL=1:8bit interface date, N=1:2lines)
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x27, ack);
    ack = i2c_write(0x80);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(0b00001100);        // Display On (D=1:display on, C=0:cursor disapper, B=0:Blink off)
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x28, ack);

    ack = i2c_write(0x00);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x29, ack);
    ack = i2c_write(0b00000001);        // Clear Display
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    //write_eeprom(0x2A, ack);
    //delay_ms(2);			// Clear Display
    wait_ms(2);			// Clear Display

stop_condition:

    i2c_stop();
    return ret;
}


/*
 *  DDRAM address set
 */
void LCD_Locate(unsigned char line, unsigned char column){
    int8    ack;
    
    ack = i2c_write(0x80);      // Co=1, RS=0:Instruction Write
    ack = i2c_write((line==0 ? 0x80 : 0xc0) + column);      // Set DDRAM address
}

void LCD_NPrintln(const unsigned char *str, unsigned char n)
{
    int8    ack;
    unsigned char *startp;

    startp = str;

    while(*str && (uint8_t)(str - startp + 1) <= n){
        ack = i2c_write(0xC0);  // Co=1, RS=1:Data Write
        ack = i2c_write(*str++);
    }
}

void LCD_Println(const unsigned char *str)
{
    int8    ack;
    int     i = 0;

    while(*str && i < 16){
        ack = i2c_write(0xC0);
        ack = i2c_write(*str++);
        i++;
    }
}

void LCD_LPrintln(unsigned char line, const unsigned char *str){
    while (LCD_I2C_Start() == 1) ;
    LCD_Locate(line, 0); LCD_Println(str);
    i2c_stop();
}

void LCD_LCPrintln(unsigned char line, unsigned char column, const unsigned char *str){
    while (LCD_I2C_Start() == 1) ;
    LCD_Locate(line, column); LCD_Println(str);
    i2c_stop();
}

void LCD_Disp_Sd_Ng()
{
    while (LCD_I2C_Start() == 1) ;
    LCD_Locate(0, 0); LCD_Println("SD CARD ERROR   ");
    LCD_Locate(1, 0); LCD_Println("                ");
    i2c_stop();
}

void LCD_RadioMode_Line0(uint8_t FmAm, char *stname){
    while (LCD_I2C_Start() == 1) ;
    LCD_Locate(0, 0);
    if (FmAm == 1) LCD_Println("FM:");
    else LCD_Println("AM:");
    LCD_Println(stname);
    LCD_Println(" ");
    i2c_stop();
}

void LCD_Recording_Line1(char *label){
    char                CurTime[8];
    
    RTC_GetCurrentTimeHHMMSS(CurTime);  // This uses I2C.
    
    while (LCD_I2C_Start() == 1) ;
    LCD_Locate(1, 0);    
    LCD_NPrintln(label, 10);
    LCD_Println(CurTime);
    i2c_stop();    
}

void LCD_Icon_RadioMode(BOOLEAN on){
    int8    ack;
    
    while (LCD_I2C_Start() == 1) ;
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x40);      // Command : Set ICON address (00H : S1 : Anntena icon)

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (on == 1){
        ack = i2c_write(0x10);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x00
    }

    i2c_stop();    
}

void LCD_Icon_PlaybackMode(BOOLEAN on){
    int8    ack;
    
    while (LCD_I2C_Start() == 1) ;
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x44);      // Command : Set ICON address (04H : S21 : Sound icon)

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (on == 1){
        ack = i2c_write(0x10);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x00
    }

    i2c_stop();    
}

void LCD_Icon_EncodeDecode(BOOLEAN on){
    int8    ack;
    
    while (LCD_I2C_Start() == 1) ;
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x47);      // Command : Set ICON address (04H : S21 : Sound icon)

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (on == 1){
        ack = i2c_write(0x18);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x00
    }

    i2c_stop();    
}

#if 0
void lcd_icon_for_subdirectory(uint16_t numsndfile, uint16_t recnosubdir)
{
    int8    ack;

    LCD_I2C_Start();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x44);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (numsndfile != 0){
        ack = i2c_write(0x10);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x00
    }

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x47);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (recnosubdir > 1){
        ack = i2c_write(0x08);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x10
    }
    
    i2c_stop();
}

/*
 *  Bass/Treble Control state icon
 */
void lcd_icon_bt_control(uint8_t bt_control)
{
    int8        ack;
    uint8_t     state;

    LCD_I2C_Start();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x4D);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (bt_control >= 1) state = 0x02;  // display "battery frame"
    else state = 0x00;
    if (bt_control >= 1) state |= 0x10;
    if (bt_control >= 2) state |= 0x08;
    if (bt_control >= 3) state |= 0x04;
    ack = i2c_write(state);      // Data    :

    i2c_stop();
}
#endif

#if 0
void lcd_set_icon()
{
    int8    ack;
    unsigned char i;

    LCD_I2C_Start();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    for (i=0; i<16; i++){
        ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
        ack = i2c_write(0x40 + i);      // Command : Set ICON address

        ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
        if (i == 0){
            ack = i2c_write(0x10);      // Data    : 0x10
        } else {
            ack = i2c_write(0x00);      // Data    : 0x00
        }
    }
    
    i2c_stop();
}
#endif

