/* 
 * File:   m41t62.c
 * 
 * Created on 2015/10/13
 *
 * Real Time Clock M41T62
 */


#define RTC_ADDRESS         0xD0            // 1101 000 0
#define RTC_ADDRESS_R       (RTC_ADDRESS | 0x01)            // 1101 000 1

static uint8_t     RtcBuff[16];

/*-----------------------------------------------------------------------*/
/* Write to M41T62                                                       */
/*-----------------------------------------------------------------------*/
uint8_t RTC_Write(uint8_t startaddress, uint8_t numbytes, uint8_t bytes[]){
    int8        ack;
    uint8_t     i;
    uint8_t     ret = 0;

    i2c_start();
    ack = i2c_write(RTC_ADDRESS);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    write_eeprom(0x40, ack);		// for debug
//    delay_ms(10);
    ack = i2c_write(startaddress);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    write_eeprom(0x41, ack);
//    delay_ms(10);
    for (i=0; i<numbytes; i++){
        ack = i2c_write(bytes[i]);
        if (ack == 1){
            ret = 1;
            goto stop_condition;
        }
//        write_eeprom(0x42 + i, ack);
//        delay_ms(10);
    }

stop_condition:
    i2c_stop();
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Read from M41T62                                                       */
/*-----------------------------------------------------------------------*/
uint8_t RTC_Read(uint8_t startaddress, uint8_t numbytes, uint8_t bytes[]){
    int8        ack;
    uint8_t     i;
    uint8_t     ret = 0;

    i2c_start();
    ack = i2c_write(RTC_ADDRESS);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(10);
    ack = i2c_write(startaddress);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(10);
    i2c_start();
    ack = i2c_write(RTC_ADDRESS_R);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
//    delay_ms(10);
    for (i=0; i<numbytes; i++){
        //bytes[i] = i2c_read(1);     // ACK
        if (i < (numbytes - 1)){
            bytes[i] = i2c_read(1);     // ACK
        } else {
            bytes[i] = i2c_read(0);     // NO ACK
        }
//        delay_ms(10);
    }

stop_condition:
    i2c_stop();
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Reset M41T62                                                          */
/*-----------------------------------------------------------------------*/
uint8_t RTC_Reset(){
    RtcBuff[0] = 0x80;      // 01h -> ST(stop bit) on
    while (RTC_Write(0x01, 1, RtcBuff) != 0);

    delay_ms(10);

    RtcBuff[0] = 0x00;      // 01h -> ST off
    while (RTC_Write(0x01, 1, RtcBuff) != 0);

    delay_ms(10);

    RtcBuff[0] = 0x00;      // 00h -> 0.1sec 0.01sec
    RtcBuff[1] = 0x00;      // 01h -> second
    RtcBuff[2] = 0x00;      // 02h -> minute
    RtcBuff[3] = 0x12;      // 03h -> hour
    RtcBuff[4] = 0x01;      // 04h -> day of week
    RtcBuff[5] = 0x01;      // 05h -> day of month
    RtcBuff[6] = 0x11;      // 06h -> month
    RtcBuff[7] = 0x15;      // 07h -> year
    while (RTC_Write(0x00, 8, RtcBuff) != 0) ;

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Init M41T62                                                           */
/*-----------------------------------------------------------------------*/
uint8_t RTC_Init(){
    while (RTC_Read(0x0A, 1, RtcBuff) != 0) ;

    RtcBuff[0] &= 0xBF;         // SQWE off

    while (RTC_Write(0x0A, 1, RtcBuff) != 0) ;
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Set Date and Time to M41T62                                           */
/*    datetime : YYMMDDWHHMMSS                                           */
/*        W = day of week (1 - 7)                                        */
/*-----------------------------------------------------------------------*/
uint8_t RTC_SetDateTime(char *datetime){
	
    RtcBuff[0] = 0x00;      // 00h -> 0.1sec 0.01sec
    RtcBuff[1] = (uint8_t)((datetime[11] - '0') << 4) | (datetime[12] - '0');		// 01h -> second
    RtcBuff[2] = (uint8_t)((datetime[9] - '0') << 4) | (datetime[10] - '0');		// 02h -> minute
    RtcBuff[3] = (uint8_t)((datetime[7] - '0') << 4) | (datetime[8] - '0');		// 03h -> hour
    RtcBuff[4] = (datetime[6] - '0');							// 04h -> day of week
    RtcBuff[5] = (uint8_t)((datetime[4] - '0') << 4) | (datetime[5] - '0');		// 05h -> day of month
    RtcBuff[6] = (uint8_t)((datetime[2] - '0') << 4) | (datetime[3] - '0');		// 06h -> month
    RtcBuff[7] = (uint8_t)((datetime[0] - '0') << 4) | (datetime[1] - '0');		// 07h -> year
    
    while (RTC_Write(0x00, 8, RtcBuff) != 0) ;

    return 0;
}


/*-----------------------------------------------------------------------*/
/* Current Date YYMMDD                                                   */
/*-----------------------------------------------------------------------*/
uint8_t RTC_GetCurrentDateYYMMDD(char *buff){
    while (RTC_Read(0x00, 8, RtcBuff) != 0) ;
    
    buff[0] = itoc((RtcBuff[7] >> 4) & 0x0F);		// YY
    buff[1] = itoc(RtcBuff[7] & 0x0F);
    buff[2] = itoc((RtcBuff[6] >> 4) & 0x03);		// MM
    buff[3] = itoc(RtcBuff[6] & 0x0F);
    buff[4] = itoc((RtcBuff[5] >> 4) & 0x03);		// DD
    buff[5] = itoc(RtcBuff[5] & 0x0F);
    buff[6] = '\0';
    
    return 0;
}

/*-------------------------------------------------------------------*/
/* Current Time HHMMSS                                                   */
/*-----------------------------------------------------------------------*/
uint8_t RTC_GetCurrentTimeHHMMSS(char *buff){
    //while (RTC_Read(0x01, 3, RtcBuff) != 0) ;     // This cannot get hour-min-sec ?!
    while (RTC_Read(0x00, 8, RtcBuff) != 0) ;
    
    buff[0] = itoc((RtcBuff[3] >> 4) & 0x03);		// HH
    buff[1] = itoc(RtcBuff[3] & 0x0F);
    buff[2] = itoc((RtcBuff[2] >> 4) & 0x07);		// MM
    buff[3] = itoc(RtcBuff[2] & 0x0F);
    buff[4] = itoc((RtcBuff[1] >> 4) & 0x07);		// SS
    buff[5] = itoc(RtcBuff[1] & 0x0F);
    buff[6] = '\0';
    
    return 0;
}

/*-------------------------------------------------------------------*/
/* Current Date&Time YYMMDDhhmmss                                        */
/*-----------------------------------------------------------------------*/
uint8_t RTC_GetCurrentDateTimeBCD(uint8_t *dt){
    while (RTC_Read(0x00, 8, RtcBuff) != 0) ;

    dt[5] = (RtcBuff[1] & 0x7F);    // sec
    dt[4] = (RtcBuff[2] & 0x7F);    // min
    dt[3] = (RtcBuff[3] & 0x3F);    // hour
    dt[2] = (RtcBuff[5] & 0x3F);    // day
    dt[1] = (RtcBuff[6] & 0x1F);    // month
    dt[0] = RtcBuff[7];             // year

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Current Date YYMMDDWHHMMSS                                            */
/*-----------------------------------------------------------------------*/
uint8_t RTC_GetCurrentDateTimeForAdjust(char *buff){
    while (RTC_Read(0x00, 8, RtcBuff) != 0) ;

    buff[0] = itoc((RtcBuff[7] >> 4) & 0x0F);		// YY
    buff[1] = itoc(RtcBuff[7] & 0x0F);
    buff[2] = itoc((RtcBuff[6] >> 4) & 0x03);		// MM
    buff[3] = itoc(RtcBuff[6] & 0x0F);
    buff[4] = itoc((RtcBuff[5] >> 4) & 0x03);		// DD
    buff[5] = itoc(RtcBuff[5] & 0x0F);
    buff[6] = itoc(RtcBuff[4] & 0x07);                  // day of week
    buff[7] = itoc((RtcBuff[3] >> 4) & 0x03);		// HH
    buff[8] = itoc(RtcBuff[3] & 0x0F);
    buff[9] = itoc((RtcBuff[2] >> 4) & 0x07);		// MM
    buff[10] = itoc(RtcBuff[2] & 0x0F);
    buff[11] = itoc((RtcBuff[1] >> 4) & 0x07);		// SS
    buff[12] = itoc(RtcBuff[1] & 0x0F);
    buff[13] = '\0';
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Current Date YY/MM/DD HH:MM                                           */
/*-----------------------------------------------------------------------*/
uint8_t RTC_GetCurrentDateTimeForDisp(char *buff){
    while (RTC_Read(0x00, 8, RtcBuff) != 0) ;

    buff[0] = itoc((RtcBuff[7] >> 4) & 0x0F);		// YY
    buff[1] = itoc(RtcBuff[7] & 0x0F);
    buff[2] = '/';
    buff[3] = itoc((RtcBuff[6] >> 4) & 0x03);		// MM
    buff[4] = itoc(RtcBuff[6] & 0x0F);
    buff[5] = '/';
    buff[6] = itoc((RtcBuff[5] >> 4) & 0x03);		// DD
    buff[7] = itoc(RtcBuff[5] & 0x0F);
    buff[8] = ' ';
    buff[9] = itoc((RtcBuff[3] >> 4) & 0x03);		// HH
    buff[10] = itoc(RtcBuff[3] & 0x0F);
    buff[11] = ':';
    buff[12] = itoc((RtcBuff[2] >> 4) & 0x07);		// MM
    buff[13] = itoc(RtcBuff[2] & 0x0F);
    buff[14] = '\0';
    //buff[15] = itoc((RtcBuff[1] >> 4) & 0x07);		// SS
    //buff[16] = itoc(RtcBuff[1] & 0x0F);
    //buff[17] = '\0';
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Set Alarm date time                                                   */
/*      YMDHMS : BCD date                                                */
/*-----------------------------------------------------------------------*/
uint8_t RTC_SetAlarm(uint8_t *YMDHMS){
    while (RTC_Read(0x0A, 6, RtcBuff) != 0) ;

    RtcBuff[0] &= 0x80;         // 0x0A : month (SQWE off)
    RtcBuff[1] &= 0xC0;         // 0x0B : date
    RtcBuff[2] &= 0x80;         // 0x0C : hour
    RtcBuff[3] &= 0x80;         // 0x0D : min
    RtcBuff[4] &= 0x80;         // 0x0E : sec
    RtcBuff[5] &= 0xFB;         // 0x0F : OF->OFF

    RtcBuff[0] |= YMDHMS[1];
    RtcBuff[1] |= YMDHMS[2];
    RtcBuff[2] |= YMDHMS[3];
    RtcBuff[3] |= YMDHMS[4];
    RtcBuff[4] |= YMDHMS[5];

    // AFE/RPT bit
    RtcBuff[0] |= 0x80;     // AFE:1
    RtcBuff[1] &= 0x3F;     // RPT4,PRT5 : 0
    RtcBuff[2] &= 0x3F;     // RPT3 : 0
    RtcBuff[3] &= 0x7F;     // RPT2 : 0
    RtcBuff[4] &= 0x7F;     // RPT1 : 0

    while (RTC_Write(0x0A, 6, RtcBuff) != 0) ;
    
    return 0;
}

uint8_t RTC_GetFlags(){
    while (RTC_Read(0x0F, 1, RtcBuff) != 0) ;
    
    return RtcBuff[0];
}


uint8_t RTC_GetAllRegisterToStr(char *buff){
    int8    i;

    while (RTC_Read(0x00, 16, RtcBuff) != 0) ;

    for (i=0; i<16; i++){
        buff[i*2] = itoH((RtcBuff[i] & 0xF0) >> 4);
        buff[i*2 + 1] = itoH(RtcBuff[i] & 0x0F);
    }
    buff[32] = '\0';
    
    return 0;
}

uint8_t RTC_GetAllRegister(uint8_t *buff){

    while (RTC_Read(0x00, 16, buff) != 0) ;

    return 0;
}

uint8_t RTC_ClearAlarm(){
    while (RTC_Read(0x0A, 6, RtcBuff) != 0) ;
    
    RtcBuff[0] = 0x00;      // 0x0A AFE:0
    RtcBuff[1] = 0x00;      // 0x0B
    RtcBuff[2] = 0x00;      // 0x0C
    RtcBuff[3] = 0x00;      // 0x0D
    RtcBuff[4] = 0x00;      // 0x0E
    RtcBuff[5] = 0x00;      // 0x0F
    
    while (RTC_Write(0x0A, 6, RtcBuff) != 0) ;
    
    return 0;
}

uint8_t RTC_STbitOnOff(){
    while (RTC_Read(0x01, 1, RtcBuff) != 0) ;
    
    RtcBuff[1] |= 0x80;      // 01h -> ST(stop bit) on
    while (RTC_Write(0x01, 1, RtcBuff) != 0);

    delay_ms(10);

    RtcBuff[1] &= 0x7F;      // 01h -> ST off
    while (RTC_Write(0x01, 1, RtcBuff) != 0);
    
    return 0;
}

uint8_t RTC_ResetFlags(){
    RtcBuff[0] = 0x00;
    
    while (RTC_Write(0x0F, 1, RtcBuff) != 0);
    
    return 0;
}

void DebugRTCAlarm(){
    int8    i;
    
    while (RTC_Read(0x0A, 6, RtcBuff) != 0) ;       // 0x0A - 0x0E
    
    for (i=0; i<6; i++){
        write_eeprom(EEPROM_DEBUG_2 + i, RtcBuff[i]);
    }
}

