/*
 * File:   si4735.c
 * Mp3_05
 *
 * Created on 2015/10/14
 */

#define SI_ADDRESS         0x22                     // 0010 001 0
#define SI_R_ADDRESS       (SI_ADDRESS | 0x01)      // 0010 001 1

#define SI_INIT_VOLUME      0x30

static uint8_t  SiCmdArgs[8];
static uint8_t  SiRespVals[15];
static uint8_t  SiRevision[8];


/*-----------------------------------------------------------------------*/
/* Send Command                                                          */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_SendCommand(uint8_t Cmd, uint8_t numofargs, uint8_t args[]){
    int8        ack;
    uint8_t     i;
    uint8_t     ret = 0;

    i2c_start();
    ack = i2c_write(SI_ADDRESS);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    ack = i2c_write(Cmd);
    if (ack == 1){
        ret = 1;
        goto stop_condition;
    }
    for (i=1; i<=numofargs; i++){
        ack = i2c_write(args[i-1]);
        if (ack == 1){
            ret = 1;
            goto stop_condition;
        }
    }
    
stop_condition:
    i2c_stop();
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Wait Response                                                         */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_WaitResponse(uint8_t *status){
    int8        ack;
    uint8_t     data;
    uint8_t     ret = 0;

    while (1){
        i2c_start();
        ack = i2c_write(SI_R_ADDRESS);
        if (ack == 0){
            data = i2c_read(0);        
        }
        i2c_stop();
        if ((data & 0x80) != 0){
            *status = data;
            break;
        } else {
        }
        wait_ms(1);
    }
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Wait Response                                                         */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_WaitResponseWithValue(uint8_t *status, uint8_t numofval, uint8_t value[]){
    int8    ack;
    uint8_t	data;
    uint8_t     i;

    while (1){
        i2c_start();
        ack = i2c_write(SI_R_ADDRESS);
        data = i2c_read(1);
        if ((data & 0x80) != 0){
            *status = data;
            for (i=1; i<=numofval; i++){
                if (i < numofval) value[i-1] = i2c_read(1);
                else value[i-1] = i2c_read(0);
            }
            i2c_stop();
            break;
        } else {
            i2c_stop();
        }
        wait_ms(1);
    }
    return 0;
}

/*-----------------------------------------------------------------------*/
/* Power Up FM                                                           */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_PowerUpFm(){
    uint8_t     status;
    SiCmdArgs[0] = 0x10;        // Use crystal oscillator / FM receive
    SiCmdArgs[1] = 0x05;        // Analog audio output

    Si4735_SendCommand(0x01, 2, SiCmdArgs);     // power up
    wait_ms(100);
    Si4735_WaitResponse(&status);

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Power Up AM                                                           */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_PowerUpAm(){
    uint8_t     status;

    SiCmdArgs[0] = 0x11;    // Use crystal oscillator
    SiCmdArgs[1] = 0x05;    // Analog audio output

    Si4735_SendCommand(0x01, 2, SiCmdArgs);     // power up 
    wait_ms(100);
    Si4735_WaitResponse(&status);

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Power down                                                            */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_PowerDown(){
    uint8_t     status;

    Si4735_SendCommand(0x11, 0, SiCmdArgs);     // power down
    wait_ms(100);
    Si4735_WaitResponse(&status);
    
    return 0;
}

/*-----------------------------------------------------------------------*/
/* RCLK Settings                                                         */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_RclkSettings(){
    uint8_t     status;

    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0x02;      // ARG2+3 : property 0x0201
    SiCmdArgs[2] = 0x01;
    SiCmdArgs[3] = 0x80;      // ARG4+5 : 32768KHz
    SiCmdArgs[4] = 0x00;
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    //delay_ms(10);
    Si4735_WaitResponse(&status);

    // 0x0202 : Sets the prescaler value for RCLK input -> uses default value

    return 0;
}

/*-----------------------------------------------------------------------*/
/* Set Volume                                                            */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_Volume(uint16_t volume){
    uint8_t     status;

    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0x40;      // ARG2+3 : property 0x4000
    SiCmdArgs[2] = 0x00;
    SiCmdArgs[3] = (volume >> 8);
    SiCmdArgs[4] = (volume & 0xFF);
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    Si4735_WaitResponse(&status);

    return 0;
}


/*-----------------------------------------------------------------------*/
/* FM Deemphasis                                                         */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_DeemphasisFm(){
    uint8_t     status;

    // Set Deemphasis (property 0x1100 FM)
    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0x11;      // ARG2+3 : property 0x1100
    SiCmdArgs[2] = 0x00;
    SiCmdArgs[3] = 0x00;      // ARG4+5 : 1=50us
    SiCmdArgs[4] = 0x01;
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    //delay_ms(10);
    Si4735_WaitResponse(&status);

    return 0;
}


/*-----------------------------------------------------------------------*/
/* AM_CHANNEL_FILTER                                                     */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_AmChannelFilter(uint16_t filter){
    uint8_t     status;

    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0x31;      // ARG2+3 : property 0x3102
    SiCmdArgs[2] = 0x02;
    SiCmdArgs[3] = (filter >> 8);      // ARG4+5 : 
    SiCmdArgs[4] = (filter & 0x00FF);
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    //delay_ms(10);
    Si4735_WaitResponse(&status);

    return 0;
}

/*-----------------------------------------------------------------------*/
/* FM Tuning                                                             */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_TuneFrequencyFm(uint16_t f_10K){
    uint8_t status;

    while(1){
        // Set FM Tune Frequency (command 0x20)
        SiCmdArgs[0] = 0x00;                        // Freeze / FAST tuning
        SiCmdArgs[1] = (f_10K >> 8);
        SiCmdArgs[2] = (f_10K & 0xFF);
        SiCmdArgs[3] = 0x00;                        // Antenna tuning capacitor
        Si4735_SendCommand(0x20, 4, SiCmdArgs);   // FM_TUNE_FREQ command
        //delay_ms(10);
        Si4735_WaitResponse(&status);
        if ((status & 0x40) != 0) continue;     // ERR

        // Use GET_INT_STATUS (command 0x14)
        while (1){
            Si4735_SendCommand(0x14, 0, SiCmdArgs);
            //delay_ms(10);
            Si4735_WaitResponse(&status);
            if ((status & 0x01) != 0){      // STCINT bit
                break;
            }
        }

        // Call FM_TUNE_STATUS With INTACK bit set (command 0x22)
        SiCmdArgs[0] = 0x01;
        Si4735_SendCommand(0x22, 1, SiCmdArgs);
        //delay_ms(10);
        Si4735_WaitResponseWithValue(&status, 7, SiRespVals);
        if (SiRespVals[1] == (f_10K >> 8) && SiRespVals[2] == (f_10K & 0xFF)){
            break;
        }
    }

    return 0;
}

/*-----------------------------------------------------------------------*/
/* AM Tuning                                                             */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_TuneFrequencyAm(uint16_t f_1K){
    //uint8_t     ack;
    uint8_t     status;
    //uint8_t     resval[15];

    while(1){
        // Set AM Tune Frequency (command 0x40 )
        i2c_start();
        SiCmdArgs[0] = 0x00;
        SiCmdArgs[1] = (f_1K >> 8);
        SiCmdArgs[2] = (f_1K & 0xFF);
        SiCmdArgs[3] = 0x00;
        Si4735_SendCommand(0x40, 4, SiCmdArgs);   // AM_TUNE_FREQ command
        Si4735_WaitResponse(&status);
        if ((status & 0x40) != 0) continue;

        // Use GET_INT_STATUS (command 0x14)
        while (1){
            Si4735_SendCommand(0x14, 0, SiCmdArgs);
            Si4735_WaitResponse(&status);
            if ((status & 0x01) != 0){      // STCINT bit
                break;
            }
        }

        // Call AM_TUNE_STATUS With INTACK bit set (command 0x22)
        SiCmdArgs[0] = 0x01;
        Si4735_SendCommand(0x42, 1, SiCmdArgs);
        Si4735_WaitResponseWithValue(&status, 7, SiRespVals);
        if (SiRespVals[1] == (f_1K >> 8) && SiRespVals[2] == (f_1K & 0xFF)){
            break;
        }

    }

    return 0;
}

/*-----------------------------------------------------------------------*/
/* FM_BLEND_RSSI_STEREO_THRESHOLD for -D60                               */
/*  0x1800: */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_FmBlendStereoThreshold(uint8_t threshold){
    
    return 0;
}

/*-----------------------------------------------------------------------*/
/* FM_BLEND_RSSI_MONO_THRESHOLD                                          */
/*  0x1801 : */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_FmBlendMonoThreshold(uint8_t threshold){
    uint8_t     status;

    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0x18;      // ARG2+3 : property 0x1801
    SiCmdArgs[2] = 0x01;
    SiCmdArgs[3] = 0x00;      // ARG4+5 : Theshold
    SiCmdArgs[4] = threshold;
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    //delay_ms(10);
    Si4735_WaitResponse(&status);

    return 0;
}


/*--------------------------------------------------------------------- -*/
/* Revision Info                                                         */
/* 1 Final 2 digits of Part Number (HEX).                                */
/* 2 Firmware Major Revision (ASCII).                                    */
/* 3 Firmware Minor Revision (ASCII).                                    */
/* 4 Patch ID High Byte (HEX).                                           */
/* 5 Patch ID Low Byte (HEX).                                            */
/* 6 Component Major Revision (ASCII).                                   */
/* 7 Component Minor Revision (ASCII).                                   */
/* 8 Chip Revision (ASCII).                                              */
/*                                                                       */
/* Result:                                                               */
/* 0x23(35),0x36('6'),0x30('0'),0x00,0x00,0x37('7'),0x30('0'),0x44('D')  */
/*-----------------------------------------------------------------------*/
uint8_t Si4735_GetRev(){
    uint8_t status;
    int8    i;
    
    Si4735_SendCommand(0x10, 0, SiCmdArgs);
    //delay_ms(10);
    Si4735_WaitResponseWithValue(&status, 8, SiRespVals);
    for (i=0; i<8; i++){
        SiRevision[i] = SiRespVals[i];
        write_eeprom(EEPROM_SI4735_REV + i , SiRevision[i]);
    }
    return 0;
}

// 0x12 0x00 0xFF 0x00 0x00 0x00
uint8_t Si4735_DisableFmDebugFeature(){
    uint8_t     status;

    SiCmdArgs[0] = 0x00;
    SiCmdArgs[1] = 0xFF;      // ARG2+3 : 
    SiCmdArgs[2] = 0x00;
    SiCmdArgs[3] = 0x00;      // ARG4+5 : 
    SiCmdArgs[4] = 0x00;
    Si4735_SendCommand(0x12, 5, SiCmdArgs);
    Si4735_WaitResponse(&status);
    
    return 0;
}

/*---------------------------------------------------------------------*/
/* Initialize FM                                                         */
/*-----------------------------------------------------------------------*/
void Si4735_InitFm(uint8_t BlendMonoThreshold){
    output_high(SI_RST);
    wait_ms(100);
    output_low(SI_RST);
    wait_ms(100);
    output_high(SI_RST);
    wait_ms(100);

    Si4735_PowerUpFm();
    Si4735_GetRev();
    Si4735_RclkSettings();
    Si4735_DeemphasisFm();
    Si4735_FmBlendMonoThreshold(BlendMonoThreshold);
    Si4735_DisableFmDebugFeature();
    
    // Set Soft Mute Settings (property 0x1301 0x1303)

    // Set Volume (property 0x4000)
    Si4735_Volume(SI_INIT_VOLUME);

    // Set Mute/Unmute (property 0x4001)

    // Set Max Tune Error (property 0x1108)

    wait_ms(1000);     // needs long time before tuning(?)
}

/*---------------------------------------------------------------------*/
/* Initialize AM                                                         */
/*-----------------------------------------------------------------------*/
void Si4735_InitAm(){
    output_high(SI_RST);
    wait_ms(100);
    output_low(SI_RST);
    wait_ms(100);
    output_high(SI_RST);
    wait_ms(100);

    Si4735_PowerUpAm();
    Si4735_RclkSettings();

    Si4735_AmChannelFilter(0x0101);

    // Set Mono/Stereo Blend settings (property 0x1800-0x180B)

    // Set Soft Mute Settings (property 0x1301 0x1303)

    // Set Volume (property 0x4000)
    Si4735_Volume(SI_INIT_VOLUME);

    // Set Mute/Unmute (property 0x4001)

    // Set Max Tune Error (property 0x1108)

    wait_ms(1000);     // a little long time

}

