/*
 * File:   vs1063.c
 * Mp3_05
 *
 * Created on 2015/11/04
 */

// VS1063 register constants
#define SCI_MODE		0x00
#define SCI_CLOCKF              0x03
#define SCI_AUDATA              0x05
#define SCI_WRAM                0x06
#define SCI_WRAMADDR            0x07
#define SCI_HDAT0               0x08
#define SCI_HDAT1               0x09
#define SCI_AIADDR              0x0A
#define SCI_VOL                 0x0B
#define SCI_AICTRL0             0x0C
#define SCI_AICTRL1             0x0D
#define SCI_AICTRL2             0x0E
#define SCI_AICTRL3             0x0F

// VS1063 SCI_MODE bits
#define SM_DIFF_B             	0x0001		// 0
#define SM_LAYER12_B          	0x0002		/* VS1063, VS1053, VS1033, VS1011 */
#define SM_RESET_B            	0x0004
#define SM_CANCEL_B           	0x0008		//3 /* VS1063, VS1053 */
#define SM_TESTS_B            	0x0020		// 5
#define SM_DACT_B             	0x0100
#define SM_SDIORD_B           	0x0200		// 9
#define SM_SDISHARE_B         	0x0400		//10
#define SM_SDINEW_B           	0x0800		// 11
#define SM_ENCODE_B           	0x1000		// 12 /* VS1063 */
#define SM_LINE1_B            	0x4000		// 14 /* VS1063, VS1053 */
#define SM_CLK_RANGE_B        	0x8000		// 15 /* VS1063, VS1053, VS1033 */

/* Parametric */
#define PAR_END_FILL_BYTE       0x1e06          /* VS1063, VS1053 */

/* WRAMADDR : bitrate mode / bitrate multiplier */
#define RQ_MODE_QUALITY 0x0000
#define RQ_MODE_VBR     0x4000
#define RQ_MODE_ABR     0x8000
#define RQ_MODE_CBR     0xc000
#define RQ_MULT_10      0x0000
#define RQ_MULT_100     0x1000
#define RQ_MULT_1000    0x2000
#define RQ_MULT_10000   0x3000
#define RQ_MP3_NO_BIT_RESERVOIR     0x0400      // (1<<10)

#define ENCODING_GAIN   (1024*8)
#define RETRY_VS1063_MAX    10

static uint8_t      Mp3Buff[ENCODE_BUF_SIZE];

#if 0
typedef struct VS1063_Encode_Setting {
    uint16_t    clockf;         // SCI_CLOCKF value
    uint16_t    samplerate;     // SCI_AICTRL0 value
    uint16_t    bitrate;        // SCI_WRAMADDR bit8:0 value 
    uint16_t    format;         // SCI_AICTRL3 value
    //char        *disp;
} VS1063_ENCODE_SETTING;

static VS1063_ENCODE_SETTING EncodeSettings[] = {
    {0x8000, 44100, 128, 0x60},
    {0x8000, 44100,  96, 0x60},
    {0x8000, 32000,  96, 0x60},
    {0x6000, 32000,  96, 0x63}
#if 0
    {0x8000, 44100, 128, 0x60, "X3.5/44.1/128/ST"},
    {0x8000, 44100,  96, 0x60, "X3.5/44.1/ 96/ST"},
    {0x8000, 32000,  96, 0x60, "X3.5/32.0/ 96/ST"},
    {0x6000, 32000,  96, 0x63, "X3.0/32.0/ 96/RC"}
#endif
};
static uint8_t      NumEncodeSettings = (sizeof EncodeSettings / sizeof EncodeSettings[0]);
#endif


#define NUMENCODESETTINGS       6

// ^p
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0xC000, 0xC000, 0x8000, 0xA000, 0xA000, 0x8000};
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {48000,  48000,  32000,  32000,  48000,  32000 };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {112,    128,    80 ,    128,    96,     64    };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x60,   0x60,   0x60,   0x60,   0x63,   0x63  };
const uint16_t      BytesPerSec[NUMENCODESETTINGS]      = {13800,  15700,  9900,   15800,  11800,  7900  };     // MP3TCY/b = ()l

// eXg

//#define NUMENCODESETTINGS       4

#if 0
// eXgp sample rate = 48000Hz / stereo
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0xC000, 0xC000, 0xE000, 0xE000 };
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {48000,  48000,  48000,  48000  };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {112,    128,    160,    192    };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x60,   0x60,   0x60,   0x60   };
#endif

#if 0
// eXgp sample rate = 44100Hz / stereo
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0xA000, 0xA000, 0xA000, 0xA000 };
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {44100,  44100,  44100,  44100  };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {96 ,    112,    128,    160    };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x60,   0x60,   0x60,   0x60   };
#endif

#if 0
// eXgp sample rate = 32000Hz / stereo
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0x8000, 0x8000, 0x8000, 0xA000 };
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {32000,  32000,  32000,  32000  };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {80 ,    96,     112,    128    };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x60,   0x60,   0x60,   0x60   };
#endif

#if 0
// eXgp sample rate = 48000Hz / mono
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0x8000, 0x8000, 0xA000, 0xA000 };
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {48000,  48000,  48000,  48000  };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {64 ,    80 ,    96 ,    128    };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x63,   0x63,   0x63,   0x63   };
#endif

#if 0
// eXgp sample rate = 32000Hz / mono
const uint16_t      ClockfTab[NUMENCODESETTINGS]        = {0x8000, 0x8000, 0x8000, 0x8000 };
const uint16_t      SamplerateTab[NUMENCODESETTINGS]    = {32000,  32000,  32000,  32000  };
const uint16_t      BitrateTab[NUMENCODESETTINGS]       = {56 ,    64 ,    80 ,    96     };
const uint16_t      EncFormatTab[NUMENCODESETTINGS]     = {0x63,   0x63,   0x63,   0x63   };
#endif


// Encode Seting vs Stereo Mono Mode
#define GetStereoModeOfEncSet(E)       ((EncFormatTab[(E)] & 0x07) <= 1 ? 1 : 0)        // ADC mode (VS1063)<=1 : stereo


/*-----------------------------------------------------------------------*/
/* Hardware Reset VS1063                                                 */
/*-----------------------------------------------------------------------*/
void VS1063_HardwareReset(){

    output_low(MP3_RESET);
    output_high(MP3_CS);
    output_high(MP3_DCS);
    //wait_ms(5000);                 // OK
    //wait_ms(1000);                 // OK
    //wait_ms(500);                 // OK
    wait_ms(250);                 // OK
    output_high(MP3_RESET);

    while(!input(MP3_DREQ));
    //mode = SM_SDINEW_B;
    VS1063_WriteSci(SCI_MODE, SM_SDINEW_B);

    VS1063_SetVolume(Volume);
    write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
}

/*-----------------------------------------------------------------------*/
/* Software Reset VS1063                                                 */
/*-----------------------------------------------------------------------*/
void VS1063_SoftwareReset(){
    uint16_t	mode,val;

    // Dummy read
    while(!input(MP3_DREQ));
    val = VS1063_ReadSci(SCI_MODE);

    // software reset
    while(!input(MP3_DREQ));
    mode = SM_SDINEW_B | SM_RESET_B;
    VS1063_WriteSci(SCI_MODE, mode);

    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_HDAT0, 0xABAD);
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_HDAT1, 0x1DEA);
    while(!input(MP3_DREQ));
    if (VS1063_ReadSci(SCI_HDAT0) != 0xABAD || VS1063_ReadSci(SCI_HDAT1) != 0x1DEA){
    }

    VS1063_SetVolume(Volume);
    write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
}

/*-----------------------------------------------------------------------*/
/* Decode a MP3 File                                                     */
/*-----------------------------------------------------------------------*/
int8 VS1063_Decode(char *Mp3FileName, FIL *file, uint16_t bytespersec){
    UINT        bytes, sendbytes, sentbytes;
    uint8_t     i;
    FRESULT     res;
    int8        ret;
    uint16_t	mode;
    int         continue_flag;
    uint32_t    skipsize;
    uint32_t    pos;
    
    skipsize = (uint32_t)bytespersec * 60;
    skipsize = (skipsize / ENCODE_BUF_SIZE) * ENCODE_BUF_SIZE;
    if (skipsize == 0) skipsize = ENCODE_BUF_SIZE;
    
    LCD_Icon_EncodeDecode(1);
    
    while(!input(MP3_DREQ));
    mode = SM_SDINEW_B;
    VS1063_WriteSci(SCI_MODE, mode);

    while(!input(MP3_DREQ));
    mode = 0x4000;      // x2.5
    //mode = 0x2000;      // enough to decode MP3 (x2.0)
    //mode = 0x1000;      // too slow
    VS1063_WriteSci(SCI_CLOCKF, mode);          // CLOCKF
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation
    
    ret = 0;
    continue_flag = 1;
    res = f_open(file, Mp3FileName, FA_READ | FA_OPEN_EXISTING);
    while (continue_flag){
        res = f_read(file, Mp3Buff, ENCODE_BUF_SIZE, &bytes);
        if (res != FR_OK || bytes == 0) break;
        sentbytes = 0;
        while (bytes > 0){
            sendbytes = (bytes > 32 ? 32 : bytes);
            while(!input(MP3_DREQ));
            output_low(MP3_DCS);
            for (i=0; i<sendbytes; i++){
                spi_write2(Mp3Buff[sentbytes + i]);
            }
            output_high(MP3_DCS);
            bytes -= sendbytes;
            sentbytes += sendbytes;
        }

        // check SW
        if (EVENT_SWANY >= 0x01 && CHATTERING_TMR == 0){
            switch(EVENT_SWANY){
            case EVENT_SWPLAYREC:    // Stop
                EVENT_SWANY = 0;
                continue_flag = 0;
                break;
            
            case EVENT_RECPREP:     // 
                EVENT_SWANY = 0;
                ret = CHANGE_TO_RADIO_REC;
                continue_flag = 0;
                break;
                    
            case EVENT_SWPOFF:       // power off
                EVENT_SWANY = 0;
                ret = SWPOFF_PRESSED;
                continue_flag = 0;
                break;
                
            case EVENT_SWUP:         // volume up
                EVENT_SWANY = 0;
                if (Volume > VOLUME_STEP){
                    Volume -= VOLUME_STEP;
                    VS1063_SetVolume(Volume);
                    write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
                }
                break;
                
            case EVENT_SWDOWN:       // volume down
                EVENT_SWANY = 0;
                if (Volume < (0x00FF - VOLUME_STEP)){
                    Volume += VOLUME_STEP;
                    VS1063_SetVolume(Volume);
                    write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
                }
             
            case EVENT_SWNEXT:
                EVENT_SWANY = 0;
                pos = f_tell(file);
                pos += skipsize;
                if (f_lseek(file, pos) != FR_OK){
                    continue_flag = 0;
                };
                break;
                
            case EVENT_SWPREV:
                EVENT_SWANY = 0;
                pos = f_tell(file);
                if (pos > skipsize){
                    pos += skipsize;
                    f_lseek(file, pos);
                }
                break;
                
            default:
                EVENT_SWANY = 0;                    
            }
        }
    }
    f_close(file);
    
    LCD_Icon_EncodeDecode(0);
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Monitor (No Encoding) Mode                                            */
/*-----------------------------------------------------------------------*/
int8 VS1063_MonitorMode(){
    uint16_t	mode,val;
    int8        ret;
    int8        i;
    //char        buff[8];
    
    ret = FR_OK;

    WriteLog("MonitorMode start", 0);
    
    //while(!input(MP3_DREQ));
    // Sometimes DREQ does not response(!?).
    i = 0;
    do {
        if (input(MP3_DREQ)) break;
        wait_ms(1);
    } while (++i < RETRY_VS1063_MAX);
    if (i == RETRY_VS1063_MAX) return NO_RESPONSE_VS1063;
    //itoa(i, 10, buff);
    //WriteLog(buff, 0);
    
    mode = 0x0000;
    VS1063_WriteSci(SCI_CLOCKF, mode);         // CLOCKF

    //WriteLog("SCI_CLOCKF", 0);
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AUDATA, 0x0010);

    //WriteLog("SCI_AUDATA", 0);
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AICTRL1, ENCODING_GAIN);        // encoding gain

    //WriteLog("SCI_AICTRL1", 0);
    
    while(!input(MP3_DREQ));
    val = VS1063_ReadSci(SCI_MODE);
    val |= (SM_ENCODE_B | SM_LINE1_B);
    VS1063_WriteSci(SCI_MODE, val);

    //WriteLog("SCI_MODE", 0);
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation

    //WriteLog("SCI_AIADDR", 0);
    
    return ret;
}

/*-----------------------------------------------------------------------*/
/* Encode a MP3 File                                                     */
/*-----------------------------------------------------------------------*/
int8 VS1063_Encode(char *Mp3FileName, FIL *file, uint8_t CurEncSet, uint16_t recno){
    UINT        fillwords, bw, sumwords;
    uint8_t     *WriteBuf;
    uint8_t     *WriteBufEnd;
    uint16_t    i;
    uint16_t    val;
    uint16_t    mode;
    FRESULT     res;
    int         continue_flag;
    uint16_t    samplerate;
    uint16_t    aictrl3;
    uint16_t    wramaddr;
    DWORD       prealloc;
    DWORD       filesize;
    int8        ret;
    uint16_t    chattering;
    uint16_t    chattering_f;
    uint8_t     swchk;

    ret = FR_OK;

    WriteLog(" VS1063_Encode start ", 1);
        
    LCD_Icon_EncodeDecode(1);
    if (!(CurEncSet >= 0 && CurEncSet <= (NUMENCODESETTINGS - 1))){
        CurEncSet = 0;
    }
    
    // chattering factor (800ms)
    chattering_f = (8 * (BytesPerSec[CurEncSet] / 10)) / ENCODE_BUF_SIZE;
    //WriteLog("LCD_Icon_EncodeDecode", 0);
    
    while(!input(MP3_DREQ));
    //mode = 0x2000;      // enough to decode MP3
    //mode = 0x6000;      // enough for mono encoding
    //mode = 0x8000;        // experiment
    //mode = 0xa000;        // experiment
    //mode = 0xe000;      // full spec.
    mode = ClockfTab[CurEncSet];
    VS1063_WriteSci(SCI_CLOCKF, mode);          // CLOCKF

    //WriteLog("SCI_CLOCKF", 0);
    
    while(!input(MP3_DREQ));
    //samplerate = 48000UL;
    //samplerate = 44100UL;
    //samplerate = 32000UL;
    //samplerate = 24000UL;
    //samplerate = 16000UL;
    samplerate = SamplerateTab[CurEncSet];
    VS1063_WriteSci(SCI_AICTRL0, samplerate);

    //WriteLog("SCI_AICTRL0", 0);
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AICTRL1, ENCODING_GAIN);           // encoding gain

    //WriteLog("SCI_AICTRL1", 0);
    
    while(!input(MP3_DREQ));
    //aictrl3 = 0x60;         // mp3/stereo
    //aictrl3 = 0x64;         // mp3/mono
    //aictrl3 = 0x63;         // mp3/right channel
    aictrl3 = EncFormatTab[CurEncSet];
    VS1063_WriteSci(SCI_AICTRL3, aictrl3);

    //WriteLog("SCI_AICTRL3", 0);
    
    /*
     *  OSC=64M/SCI_CLOCKF=0x6000/SPI1=DIV16/SPI2=DIV64
     *      mp3/stereo/SR=16K/BR=64K : OK
     *      mp3/stereo/SR=32K/BR=80K : NG
     *      mp3/mono/SR=32K/BR=64K : OK
     *  OSC=32M/SCI_CLOCKF=0x6000/SPI1=DIV4/SPI2=DIV16
     *      mp3/stereo/SR=16K/BR=64K : OK
     *      mp3/mono/SR=32K/BR=64K : OK
     *  OSC=32M/SCI_CLOCKF=0x6000/SPI1=DIV4/SPI2=DIV16/pre-allocation
     *      mp3/mono/SR=32K/BR=64K : OK
     *      mp3/mono/SR=48K/BR=80K : OK
     *      mp3/mono/SR=48K/BR=96K : OK
     *      mp3/mono/SR=48K/BR=128K : OK
     *      mp3/mono/SR=48K/BR=160K : OK
     *      mp3/stereo/SR=32K/BR=64K : NG
     *      mp3/stereo/SR=24K/BR=64K : OK
     *  OSC=32M/SCI_CLOCKF=0xe000/SPI1=DIV4/SPI2=DIV16/pre-allocation
     *      mp3/stereo/SR=32K/BR=80K : OK
     *      mp3/stereo/SR=32K/BR=96K : OK
     *      mp3/stereo/SR=44.1K/BR=96K : OK
     *      mp3/stereo/SR=44.1K/BR=128K : OK (target)
     *      mp3/stereo/SR=48K/BR=160K : OK
     */
    while(!input(MP3_DREQ));
    wramaddr = 0;
    wramaddr |= RQ_MODE_CBR;
    //wramaddr |= RQ_MODE_VBR;
    wramaddr |= RQ_MULT_1000;
    //wramaddr |= 64;
    //wramaddr |= 80;
    //wramaddr |= 96;
    //wramaddr |= 128;
    //wramaddr |= 160;
    wramaddr |= BitrateTab[CurEncSet];
    VS1063_WriteSci(SCI_WRAMADDR, wramaddr);

    //WriteLog("SCI_WRAMADDR", 0);
    
    while(!input(MP3_DREQ));
    val = VS1063_ReadSci(SCI_MODE);
    //WriteLog("READ SCI_MODE", 0);
    val |= (SM_ENCODE_B | SM_LINE1_B);
    VS1063_WriteSci(SCI_MODE, val);

    //WriteLog("WRITE SCI_MODE", 0);
    
    while(!input(MP3_DREQ));
    VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation

    //WriteLog("SCI_AIADDR", 0);
    
    res = f_open(file, Mp3FileName, FA_WRITE | FA_CREATE_NEW);
    if (res != FR_OK){
        WriteLog("f_open encode outfile error", 0);
        goto VS1063_Encode_End;
    }
    prealloc = 10000000;
    res = f_lseek(file, prealloc);
    if (res != FR_OK || f_tell(file) != prealloc){
        WriteLog("f_lseek error", 0);
        goto VS1063_Encode_End;
    }
    res = f_lseek(file, 0);

    if (res == FR_OK){
        WriteLog("recording write start", 0);

        continue_flag = 1;
        WriteBuf = &Mp3Buff[0];
        sumwords = 0;
        chattering = 0;
        swchk = 0;
        
        disable_interrupts(GLOBAL);
        
        WriteBufEnd = &Mp3Buff[0] + ENCODE_BUF_SIZE;
        while (1){
            while(!input(MP3_DREQ)) ;
            fillwords = VS1063_ReadSci(SCI_HDAT1);
            for (i=0; i<fillwords; i++){
                while(!input(MP3_DREQ));        // can omit this ?
                val = VS1063_ReadSci(SCI_HDAT0);
                *WriteBuf++ = (val >> 8);
                *WriteBuf++ = (val & 0x00FF);
                //sumwords++;
                if (WriteBuf == WriteBufEnd){
                //if (sumwords == ENCODE_BUF_SIZE_HALF){
                    res = f_write(file, Mp3Buff, ENCODE_BUF_SIZE, &bw);
                    /**
                    if (res != FR_OK){      // can omit this ?
                        WriteLog("f_write error ", 0);
                        continue_flag = 0;
                        break;
                    }
                    **/
                    WriteBuf = &Mp3Buff[0];
                    //sumwords = 0;
                    swchk = 1;
                }
            }
            /**
            if (continue_flag == 0){
                break;
            }
            **/

            if (swchk == 1){
                if (sw_playrec() == 0 && chattering == 0){      // Stop recording by switch
                    continue_flag = 0;
                    ret = SWPLAYREC_PRESSED;
                    // finalizing recording
                    while(!input(MP3_DREQ));
                    VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                    chattering = 1;
                    wait_ms(1000);
                    break;
                } else if (sw_up() == 0 && chattering == 0){
                    if (Volume > VOLUME_STEP){
                        Volume -= VOLUME_STEP;
                        VS1063_SetVolume(Volume);
                    }
                    chattering = 1;
                }
                else if (sw_down() == 0 && chattering == 0){
                    if (Volume < (0x00FF - VOLUME_STEP)){
                        Volume += VOLUME_STEP;
                        VS1063_SetVolume(Volume);
                    }
                    chattering = 1;
                } else if (sw_detectpoff() == 0 && chattering == 0){
                    int8    retrtc = OnRTCAlarm();
                    if (retrtc == 3){                              // Stop recording by RTC
                        continue_flag = 0;
                        ret = RECSTOP_ALARMED;
                        // finalizing recording
                        while(!input(MP3_DREQ));
                        VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                        wait_ms(1000);
                        break;
                    } else if (retrtc == 0){ 
                        continue_flag = 0;
                        ret = SWPOFF_PRESSED;
                        // finalizing recording
                        while(!input(MP3_DREQ));
                        VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                        break;
                    }
                    chattering = 1;
                } else if (chattering != 0){
                    if (++chattering == chattering_f) chattering = 0;
                }
                swchk = 0;
            }
            
#if 0
            // check SW
            if (EVENT_SWANY != 0x00 && CHATTERING_TMR == 0){
                switch (EVENT_SWANY){
                case EVENT_SWPLAYREC:           // Stop recording by switch
                    EVENT_SWANY = 0;
                    continue_flag = 0;
                    ret = SWPLAYREC_PRESSED;
                    // finalizing recording
                    while(!input(MP3_DREQ));
                    VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                    break;
                    
                case EVENT_RECSTOP:             // Stop recording by RTC
                    EVENT_SWANY = 0;
                    continue_flag = 0;
                    ret = RECSTOP_ALARMED;
                    // finalizing recording
                    while(!input(MP3_DREQ));
                    VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                    break;
                    
                case EVENT_SWPOFF:              // power off
                    EVENT_SWANY = 0;
                    continue_flag = 0;
                    ret = SWPOFF_PRESSED;
                    // finalizing recording
                    while(!input(MP3_DREQ));
                    VS1063_WriteSci(SCI_MODE, VS1063_ReadSci(SCI_MODE) | SM_CANCEL_B);
                    break;
                    
                case EVENT_SWUP:                // volume up
                    EVENT_SWANY = 0;
                    if (Volume > VOLUME_STEP){
                        Volume -= VOLUME_STEP;
                        VS1063_SetVolume(Volume);
                    }
                    break;
                    
                case EVENT_SWDOWN:              // volume down
                    EVENT_SWANY = 0;
                    if (Volume < (0x00FF - VOLUME_STEP)){
                        Volume += VOLUME_STEP;
                        VS1063_SetVolume(Volume);
                    }
                    break;
                    
                default:
                    // Ignore EVENT_RECPREP,EVENT_RECSTART
                    EVENT_SWANY = 0;
                    break;
                }
                
                if (continue_flag == 0){
                    break;
                }
            }
#endif
        }

        enable_interrupts(GLOBAL);
        
        while(!input(MP3_DREQ)) ;
        fillwords = VS1063_ReadSci(SCI_HDAT1);
        for (i=0; i<fillwords; i++){
            while(!input(MP3_DREQ));        // can omit this ?
            val = VS1063_ReadSci(SCI_HDAT0);
            *WriteBuf++ = (val >> 8);
            *WriteBuf++ = (val & 0x00FF);
            //sumwords++;
            if (WriteBuf == WriteBufEnd){
            //if (sumwords == ENCODE_BUF_SIZE_HALF){
                res = f_write(file, Mp3Buff, ENCODE_BUF_SIZE, &bw);
                /**
                if (res != FR_OK){      // can omit this ?
                    WriteLog("f_write error ", 0);
                    continue_flag = 0;
                    break;
                }
                **/
                WriteBuf = &Mp3Buff[0];
                //sumwords = 0;
            }
        }
        /**
        if (sumwords > 0){
            res = f_write(file, Mp3Buff, sumwords * 2, &bw);
        }
        **/
        val = VS1063_ReadWram(PAR_END_FILL_BYTE);
        if (val & 0x8000){
            f_write(file, &val, 1, &bw);
        }

        f_truncate(file);
        filesize = f_size(file);        // bytes
        f_close(file);

        SaveMp3Filesize(file, recno, filesize);
        
        write_eeprom(EEPROM_LAST_VOLUME, Volume & 0x00FF);
    } else {
    }

VS1063_Encode_End:
    LCD_Icon_EncodeDecode(0);
    WriteLog("VS1063_Encode end ", 0);
    return ret;
}

uint16_t VS1063_ReadSci(uint8_t addr){
    uint16_t    val = 0;

    output_low(MP3_CS);
    spi_write2(0x03);
    spi_write2(addr);
    val = spi_read2(0x00);
    val <<= 8;
    val |= spi_read2(0x00);
    output_high(MP3_CS);
    return val;
}

void VS1063_WriteSci(uint8_t addr, uint16_t val){
    output_low(MP3_CS);
    spi_write2(2);
    spi_write2(addr);
    spi_write2(val >> 8);
    spi_write2(val & 0x00FF);
    output_high(MP3_CS);
}

uint16_t VS1063_ReadWram(uint16_t addr){
    VS1063_WriteSci(SCI_WRAMADDR, addr);
    return VS1063_ReadSci(SCI_WRAM);
}

void VS1063_SetVolume(uint16_t volume){
    uint16_t    vol;

    while(!input(MP3_DREQ));
    vol = (volume << 8) | volume;
    VS1063_WriteSci(SCI_VOL, vol);          // VOL
}
